# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from zope.component import getMultiAdapter

import datetime


class RegisteredView(BrowserView):

    def expire_date(self):
        ppr = getToolByName(self.context, 'portal_password_reset')
        expire_length = datetime.timedelta(days=ppr.getExpirationTimeout())
        expiration_date = datetime.datetime.now() + expire_length
        ploneview = getMultiAdapter((self.context, self.request), name='plone')
        return ploneview.toLocalizedTime(expiration_date, long_format=1)



# -*- coding: utf-8 -*-
from plone.uuid.interfaces import IUUIDGenerator
from z3c.form.action import ActionErrorOccurred
from z3c.form.interfaces import WidgetActionExecutionError
from zope.component import getUtility
from zope.interface import Invalid

import zope.event


def uuid_userid_generator(data=None):
    # Generate a unique user id.  This can be used as
    # IUserIdGenerator if wanted.
    generator = getUtility(IUUIDGenerator)
    return generator()


def notifyWidgetActionExecutionError(action, widget, err_str):
    zope.event.notify(
        ActionErrorOccurred(
            action,
            WidgetActionExecutionError(widget, Invalid(err_str))
        )
    )



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces.controlpanel import IPloneControlPanelForm
from zope.interface import Interface


class IAccountPanelForm(IPloneControlPanelForm):
    """Forms using plone.app.users
    """

    def _on_save():
        """Callback mehod which can be implemented by control panels to
        react when the form is successfully saved. This avoids the need
        to re-define actions only to do some additional notification or
        configuration which cannot be handled by the normal schema adapter.

        By default, does nothing.
        """


class IUserIdGenerator(Interface):
    """Create a user id from data.

    This must be a function that accepts 'data' as an argument.
    'data' is a dictionary.  Normally it will contain keys like
    username, email, fullname, password that the user has filled in on
    the registration form.  Nothing is guaranteed though, so you
    should not assume that a key is present.

    Standard behavior would be to use the username (login name) as
    user id.  When Plone is configured to use the email address as
    login name, no username will be in the data, because the form
    will not contain that field.  Standard behavior was to use the
    email address as user id as well.  That has some downsides, as
    explained in the generate_user_id method of the registration
    view, so this was changed to create a user id based on the
    full name.

    By registering a utility for this interface, you can come up
    with a different scheme, for example to create a uuid.  Plone
    does not have such a utility, but you can look in the tests
    for an example.

    The function should return the chosen user id or None.
    """


class ILoginNameGenerator(Interface):
    """Create a login name from data.

    This must be a function that accepts 'data' as an argument.
    'data' is a dictionary.  Normally it will contain keys like
    username, email, fullname, password that the user has filled in on
    the registration form.  Nothing is guaranteed though, so you
    should not assume that a key is present.

    Standard behavior is to use the username as login name.  When
    Plone is configured to use the email address as login name, we
    take the email key.

    By registering a utility for this interface, you can come up
    with a different scheme, for example to create a uuid.  Plone
    does not have such a utility, but you can look in the tests
    for an example.

    The function should return the chosen login name or None.
    """



# -*- coding: utf-8 -*-
from pkg_resources import resource_stream
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.users.setuphandlers import import_schema
from plone.app.users.testing import PLONE_APP_USERS_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser
from Products.GenericSetup.tests.common import DummyImportContext
from plone.app.users.tests.base import BaseTestCase

import transaction


class TestSchema(BaseTestCase):

    def setUp(self):
        super(TestSchema, self).setUp()
        xml = """<model xmlns:lingua="http://namespaces.plone.org/supermodel/lingua" xmlns:users="http://namespaces.plone.org/supermodel/users" xmlns:form="http://namespaces.plone.org/supermodel/form" xmlns:i18n="http://xml.zope.org/namespaces/i18n" xmlns:security="http://namespaces.plone.org/supermodel/security" xmlns:marshal="http://namespaces.plone.org/supermodel/marshal" xmlns="http://namespaces.plone.org/supermodel/schema" i18n:domain="plone">
  <schema name="member-fields">
    <field name="home_page" type="zope.schema.URI" users:forms="In User Profile">
      <description i18n:translate="help_homepage">
          The URL for your external home page, if you have one.
      </description>
      <required>False</required>
      <title i18n:translate="label_homepage">Home Page</title>
    </field>
    <field name="description" type="zope.schema.Text" users:forms="In User Profile">
      <description i18n:translate="help_biography">
          A short overview of who you are and what you do. Will be displayed
          on your author page, linked from the items you create.
      </description>
      <required>False</required>
      <title i18n:translate="label_biography">Biography</title>
    </field>
    <field name="location" type="zope.schema.TextLine" users:forms="In User Profile">
      <description i18n:translate="help_location">
          Your location - either city and country - or in
          a company setting, where your office is located.
      </description>
      <required>False</required>
      <title i18n:translate="label_biography">Location</title>
    </field>
    <field name="portrait" type="plone.namedfile.field.NamedBlobImage" users:forms="In User Profile">
      <description i18n:translate="help_portrait">
          To add or change the portrait: click the "Browse" button; select a
          picture of yourself. Recommended image size is 75 pixels wide by
          100 pixels tall.
      </description>
      <required>False</required>
      <title i18n:translate="label_portrait">Portrait</title>
    </field>
    <field name="birthdate" type="zope.schema.Date" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Birthdate</title>
    </field>
    <field name="another_date" type="zope.schema.Datetime" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Another date</title>
    </field>
    <field name="age" type="zope.schema.Int" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Age</title>
    </field>
    <field name="department" type="zope.schema.Choice" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Department</title>
      <values>
        <element>Marketing</element>
        <element>Production</element>
        <element>HR</element>
      </values>
    </field>
    <field name="skills" type="zope.schema.Set" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Skills</title>
      <value_type type="zope.schema.Choice">
        <values>
          <element>Programming</element>
          <element>Management</element>
        </values>
      </value_type>
    </field>
    <field name="pi" type="zope.schema.Float" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Pi</title>
    </field>
    <field name="vegetarian" type="zope.schema.Bool" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Vegetarian</title>
    </field>
  </schema>
</model>
"""
        context = DummyImportContext(self.portal, purge=False)
        context._files = {'userschema.xml': xml}
        import_schema(context)
        transaction.commit()

        self.browser = Browser(self.layer['app'])
        self.request = self.layer['request']

    def test_schema_types(self):
        self.browser.open('http://nohost/plone/')
        self.browser.getLink('Log in').click()
        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl('Log in').click()
        self.browser.open("http://nohost/plone/@@personal-information")
        self.browser.getControl('Full Name').value = 'Isaac Newton'
        self.browser.getControl('Email').value = 'isaac@cambridge.com'
        self.browser.getControl('Home Page').value = 'http://gravity.org'
        self.browser.getControl('Biography').value = 'I like apples'
        self.browser.getControl('Location').value = 'Cambridge'
        portrait_file = resource_stream("plone.app.users.tests", 'onepixel.jpg')
        self.browser.getControl(name='form.widgets.portrait').add_file(portrait_file, "image/jpg", "onepixel.# jpg")
        self.browser.getControl('Age').value = '40'
        self.browser.getControl('Department').value = ['Marketing', ]
        self.browser.getControl('Skills').value = ['Programming', ]
        self.browser.getControl('Pi').value = '3.14'
        self.browser.getControl('Vegetarian').click()
        self.browser.getControl('Save').click()

        transaction.commit()
        membership = self.layer['portal'].portal_membership
        member = membership.getMemberById(TEST_USER_ID)
        self.assertTrue(isinstance(member.getProperty('fullname'), str))
        self.assertEqual(member.getProperty('fullname'), 'Isaac Newton')
        self.assertTrue(isinstance(member.getProperty('email'), str))
        self.assertEqual(member.getProperty('email'), 'isaac@cambridge.com')
        self.assertTrue(isinstance(member.getProperty('home_page'), str))
        self.assertEqual(member.getProperty('home_page'), 'http://gravity.org')
        self.assertTrue(isinstance(member.getProperty('description'), str))
        self.assertEqual(member.getProperty('description'), 'I like apples')
        self.assertTrue(isinstance(member.getProperty('location'), str))
        portrait = self.layer['portal'].portal_memberdata._getPortrait(TEST_USER_ID)
        self.assertEqual(portrait.content_type, 'image/jpeg')
        self.assertEqual(portrait.width, 1)
        self.assertEqual(portrait.height, 1)
        self.assertEqual(member.getProperty('location'), 'Cambridge')
        self.assertTrue(isinstance(member.getProperty('age'), int))
        self.assertEqual(member.getProperty('age'), 40)
        self.assertTrue(isinstance(member.getProperty('department'), str))
        self.assertEqual(member.getProperty('department'), 'Marketing')
        self.assertTrue(isinstance(member.getProperty('skills'), tuple))
        self.assertEqual(member.getProperty('skills'), ('Programming', ))
        self.assertTrue(isinstance(member.getProperty('pi'), float))
        self.assertEqual(member.getProperty('pi'), 3.14)
        self.assertTrue(isinstance(member.getProperty('vegetarian'), bool))
        self.assertEqual(member.getProperty('vegetarian'), True)



# -*- coding: utf-8 -*-
from plone.app.users.browser.schemaeditor import USERS_NAMESPACE
from plone.app.users.browser.schemaeditor import USERS_PREFIX
from plone.autoform import directives as form
from plone.supermodel.interfaces import IFieldMetadataHandler
from plone.supermodel.utils import ns
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope.component import adapts
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IField
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import zope.schema


form_vocab = SimpleVocabulary([
    SimpleTerm(value=u'On Registration',
               title=u'On Registration'),
    SimpleTerm(value=u'In User Profile',
               title=u'In User Profile'),
])


class IUserFormSelection(Interface):
    form.widget(forms=CheckBoxFieldWidget)
    forms = zope.schema.List(
        title=u"Where should this field be shown",
        description=u"Does not apply to username or to email fields",
        required=True,
        value_type=zope.schema.Choice(vocabulary=form_vocab),
    )


def get_user_form_selection(schema_context, field):
    return IUserFormSelection


def get_user_addform_selection(schema_context):
    return IUserFormSelection


class UserFormSelectionAdapter(object):
    adapts(IField)

    def __init__(self, field):
        self.field = field

    def _get_forms(self):
        forms = getattr(self.field, 'forms_selection', [])
        return forms

    def _set_forms(self, value):
        self.field.forms_selection = value

    forms = property(_get_forms, _set_forms)


@implementer(IFieldMetadataHandler)
class UserFormSelectionMetadata(object):

    namespace = USERS_NAMESPACE
    prefix = USERS_PREFIX

    def read(self, fieldNode, schema, field):
        forms = fieldNode.get(ns('forms', self.namespace))
        if forms:
            field.forms_selection = forms.split('|')

    def write(self, fieldNode, schema, field):
        forms = getattr(field, 'forms_selection', [])
        if forms:
            fieldNode.set(ns('forms', self.namespace), "|".join(forms))



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from Acquisition import aq_inner
from plone.app.users.browser.interfaces import IAccountPanelForm
from plone.app.users.utils import notifyWidgetActionExecutionError
from plone.autoform.form import AutoExtensibleForm
from plone.namedfile.file import NamedBlobImage
from plone.protect import CheckAuthenticator
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.controlpanel.events import ConfigurationChangedEvent
from Products.CMFPlone.interfaces import ISecuritySchema
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.PlonePAS.tools.membership import default_portrait
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import form
from zope import schema
from zope.cachedescriptors.property import Lazy as lazy_property
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.event import notify
from zope.interface import implementer
from ZTUtils import make_query


MESSAGE_EMAIL_CANNOT_CHANGE = \
    _('message_email_cannot_change',
      default=(u"Sorry, you are not allowed to "
               u"change your email address."))

MESSAGE_EMAIL_IN_USE = \
    _('message_email_in_use',
      default=(u"The email address you selected is "
               u"already in use or is not valid as login "
               u"name. Please choose another."))


def isDefaultPortrait(value, portal):
    default_portrait_value = getattr(portal, default_portrait, None)
    return aq_inner(value) == aq_inner(default_portrait_value)


class AccountPanelSchemaAdapter(object):
    """Data manager that gets and sets any property mentioned
       in the schema to the property sheet
    """
    context = None
    schema = IAccountPanelForm

    def __init__(self, context):
        mt = getToolByName(context, 'portal_membership')
        userid = context.REQUEST.form.get('userid')
        if (userid and mt.checkPermission('Plone Site Setup: Users and Groups',
                                          context)):
            self.context = mt.getMemberById(userid)
        else:
            self.context = mt.getAuthenticatedMember()

    def _getProperty(self, name):
        value = self.context.getProperty(name, '')
        if value == '':
            value = None
        if value:
            # PlonePAS encodes all unicode coming from PropertySheets.
            return safe_unicode(value)
        return value

    def _setProperty(self, name, value):
        if isinstance(value, set):
            value = list(value)
        if value and isinstance(self.schema[name], schema.Choice):
            value = str(value)
        return self.context.setMemberProperties(
            {name: value}, force_empty=True
        )

    def __getattr__(self, name):
        if name in self.schema:
            if isinstance(self.schema[name], NamedBlobImage):
                # any image is the portrait
                return self.get_portrait()
            # In schema and no explicit handler, assume it's in the property
            # sheet
            return self._getProperty(name)
        raise AttributeError(name)

    def __setattr__(self, name, value):
        if name not in self.schema or hasattr(self.__class__, name):
            # Either not part of the schema or dealt with by an explicit
            # property
            return super(AccountPanelSchemaAdapter, self).__setattr__(name,
                                                                      value)
        if isinstance(value, NamedBlobImage):
            # any image is stored as portrait
            return self.set_portrait(value)

        return self._setProperty(name, value)

    @property
    def portal(self):
        return getToolByName(self.context, 'portal_url').getPortalObject()

    def get_portrait(self):
        """If user has default portrait, return none"""
        mt = getToolByName(self.context, 'portal_membership')
        value = mt.getPersonalPortrait(self.context.getId())
        if isDefaultPortrait(value, self.portal):
            return None
        return NamedBlobImage(value.data, contentType=value.content_type,
                              filename=getattr(value, 'filename', None))

    def set_portrait(self, value):
        mt = getToolByName(self.context, 'portal_membership')
        member_id = self.context.getId()
        if value is None:
            previous = mt.getPersonalPortrait(member_id)
            if not isDefaultPortrait(previous, self.portal):
                mt.deletePersonalPortrait(str(member_id))
        else:
            portrait_file = value.open()
            portrait_file.filename = value.filename
            mt.changeMemberPortrait(portrait_file, str(self.context.getId()))

    portrait = property(get_portrait, set_portrait)

    @property
    def wysiwyg_editor(self):
        return self._getProperty('wysiwyg_editor')

    @wysiwyg_editor.setter
    def wysiwyg_editor(self, value):
        if value is None:
            # set property that the site-default from the registry is used
            # since both 'None' and None result in plaintexteditor
            value = ''
        return self._setProperty('wysiwyg_editor', value)

    @property
    def timezone(self):
        return self._getProperty('timezone')

    @timezone.setter
    def timezone(self, value):
        if value is None:
            value = ''
        return self._setProperty('timezone', value)


@implementer(IAccountPanelForm)
class AccountPanelForm(AutoExtensibleForm, form.Form):
    """A simple form to be used as a basis for account panel screens."""
    schema = IAccountPanelForm
    template = ViewPageTemplateFile('account-panel.pt')
    enableCSRFProtection = True

    hidden_widgets = []
    successMessage = _("Changes saved.")
    noChangesMessage = _("No changes made.")

    @lazy_property
    def member(self):
        mtool = getToolByName(self.context, 'portal_membership')
        if self.request.get('userid'):
            return mtool.getMemberById(self.request.get('userid'))
        return mtool.getAuthenticatedMember()

    @property
    def label(self):
        return self.member.getProperty('fullname') or self.member.getUserName()

    def _differentEmail(self, email):
        """Check if the submitted form email address differs from the existing
        one.

        Keeping your email the same (which happens when you change something
        else on the personalize form) or changing it back to your login name,
        is fine.
        """
        membership = getToolByName(self.context, 'portal_membership')
        if self.request.get('userid'):
            member = membership.getMemberById(self.request.get('userid'))
        else:
            member = membership.getAuthenticatedMember()
        return email not in (member.getId(), member.getUserName())

    def makeQuery(self):
        userid = self.request.form.get('userid', None)
        if userid is not None:
            return '?{}'.format(make_query({'userid': userid}))
        return ''

    def action(self):
        return self.request.getURL() + self.makeQuery()

    def validate_email(self, action, data):
        context = aq_inner(self.context)
        error_keys = [
            error.field.getName()
            for error
            in action.form.widgets.errors
        ]
        if 'email' not in error_keys:
            registration = getToolByName(context, 'portal_registration')
            registry = getUtility(IRegistry)
            security_settings = registry.forInterface(
                ISecuritySchema, prefix="plone")
            if security_settings.use_email_as_login:
                err_str = ''
                try:
                    id_allowed = registration.isMemberIdAllowed(data['email'])
                except Unauthorized:
                    err_str = MESSAGE_EMAIL_CANNOT_CHANGE
                else:
                    if not id_allowed:
                        # only allow if unchanged
                        if self._differentEmail(data['email']):
                            err_str = MESSAGE_EMAIL_IN_USE
                if err_str:
                    notifyWidgetActionExecutionError(action, 'email', err_str)

    @button.buttonAndHandler(_(u'Save'))
    def handleSave(self, action):
        CheckAuthenticator(self.request)
        data, errors = self.extractData()

        # Extra validation for email, when it is there.  email is not in the
        # data when you are at the personal-preferences page.
        if 'email' in data:
            self.validate_email(action, data)

        if action.form.widgets.errors:
            self.status = self.formErrorsMessage
            return
        if self.applyChanges(data):
            IStatusMessage(self.request).addStatusMessage(
                self.successMessage, type='info')
            notify(ConfigurationChangedEvent(self, data))
            self._on_save(data)
        else:
            IStatusMessage(self.request).addStatusMessage(
                self.noChangesMessage, type='info')
        self.request.response.redirect(self.action())

    def updateActions(self):
        super(AccountPanelForm, self).updateActions()
        if self.actions and 'save' in self.actions:
            self.actions['save'].addClass('context')

    @button.buttonAndHandler(_(u'Cancel'))
    def cancel(self, action):
        IStatusMessage(self.request).addStatusMessage(_("Changes canceled."),
                                                      type="info")
        self.request.response.redirect(
            '%s%s' % (self.request['ACTUAL_URL'], self.makeQuery())
        )

    def _on_save(self, data=None):
        pass

    def prepareObjectTabs(self,
                          default_tab='view',
                          sort_first=['folderContents']):
        context = self.context
        mt = getToolByName(context, 'portal_membership')
        tabs = []
        navigation_root_url = context.absolute_url()

        def _check_allowed(context, request, name):
            """Check, if user has required permissions on view.
            """
            view = getMultiAdapter((context, request), name=name)
            allowed = True
            for perm in view.__ac_permissions__:
                allowed = allowed and mt.checkPermission(perm[0], context)
            return allowed

        if _check_allowed(context, self.request, 'personal-information'):
            tabs.append({
                'title': _('title_personal_information_form',
                           u'Personal Information'),
                'url': navigation_root_url + '/@@personal-information',
                'selected': (self.__name__ == 'personal-information'),
                'id': 'user_data-personal-information',
            })

        if _check_allowed(context, self.request, 'personal-preferences'):
            tabs.append({
                'title': _(u'Personal Preferences'),
                'url': navigation_root_url + '/@@personal-preferences',
                'selected': (self.__name__ == 'personal-preferences'),
                'id': 'user_data-personal-preferences',
            })

        member = mt.getAuthenticatedMember()
        if member.canPasswordSet():
            tabs.append({
                'title': _('label_password', u'Password'),
                'url': navigation_root_url + '/@@change-password',
                'selected': (self.__name__ == 'change-password'),
                'id': 'user_data-change-password',
            })
        return tabs



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone.app.users.browser.account import AccountPanelForm
from plone.app.users.utils import notifyWidgetActionExecutionError
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from zope import schema
from zope.interface import Interface

import six


class IPasswordSchema(Interface):
    """Provide schema for password form """

    current_password = schema.Password(
        title=_(u'label_current_password', default=u'Current password'),
        description=_(
            u'help_current_password',
            default=u'Enter your current password.'),
        # constraint=checkCurrentPassword,
    )

    new_password = schema.Password(
        title=_(u'label_new_password', default=u'New password'),
        description=_(
            u'help_new_password',
            default=u"Enter your new password."),
    )

    new_password_ctl = schema.Password(
        title=_(u'label_confirm_password', default=u'Confirm password'),
        description=_(
            u'help_confirm_password',
            default=u"Re-enter the password. "
            u"Make sure the passwords are identical."),
    )


class PasswordPanelAdapter(object):

    def __init__(self, context):
        self.context = getToolByName(context, 'portal_membership')

    def get_dummy(self):
        """We don't actually need to 'get' anything ..."""
        return ''

    current_password = property(get_dummy)

    new_password = property(get_dummy)

    new_password_ctl = property(get_dummy)


class PasswordPanel(AccountPanelForm):
    """Implementation of password reset form that uses z3c.form."""

    description = _(u"Change Password")
    form_name = _(u'legend_password_details', default=u'Password Details')
    schema = IPasswordSchema

    def updateFields(self):
        super(PasswordPanel, self).updateFields()
        # Change the password description based on PAS Plugin The user needs a
        # list of instructions on what kind of password is required.  We'll
        # reuse password errors as instructions e.g. "Must contain a letter and
        # a number".  Assume PASPlugin errors are already translated
        registration = getToolByName(self.context, 'portal_registration')
        err_str = registration.testPasswordValidity('')
        if err_str:
            msg = _(
                u'Enter your new password. ${errors}',
                mapping=dict(errors=err_str)
            )
            self.fields['new_password'].field.description = msg

    def validate_password(self, action, data):
        context = aq_inner(self.context)
        registration = getToolByName(context, 'portal_registration')
        membertool = getToolByName(context, 'portal_membership')

        # check if password is correct
        current_password = data.get('current_password')
        if current_password:
            if six.PY2 and isinstance(current_password, six.text_type):
                current_password = current_password.encode('utf8')

            if not membertool.testCurrentPassword(current_password):
                # add error to current_password widget
                err_str = _(u"Incorrect value for current password")
                notifyWidgetActionExecutionError(action,
                                                 'current_password', err_str)

        # check if passwords are same and valid according to plugin
        new_password = data.get('new_password')
        new_password_ctl = data.get('new_password_ctl')
        if new_password and new_password_ctl:
            err_str = registration.testPasswordValidity(new_password,
                                                        new_password_ctl)

            if err_str:
                # add error to new_password widget
                notifyWidgetActionExecutionError(action,
                                                 'new_password', err_str)
                notifyWidgetActionExecutionError(action,
                                                 'new_password_ctl', err_str)

    @button.buttonAndHandler(
        _(u'label_change_password', default=u'Change Password'),
        name='reset_passwd'
    )
    def action_reset_passwd(self, action):
        data, errors = self.extractData()

        # extra password validation
        self.validate_password(action, data)

        if action.form.widgets.errors:
            self.status = self.formErrorsMessage
            return

        membertool = getToolByName(self.context, 'portal_membership')

        password = data['new_password']
        if six.PY2 and isinstance(password, six.text_type):
            password = password.encode('utf8')

        try:
            membertool.setPassword(password, None, REQUEST=self.request)
        except AttributeError:
            failMessage = _(u'While changing your password an AttributeError '
                            u'occurred. This is usually caused by your user '
                            u'being defined outside the portal.')

            IStatusMessage(self.request).addStatusMessage(
                _(failMessage), type="error"
            )
            return

        IStatusMessage(self.request).addStatusMessage(
            _("Password changed"), type="info"
        )

    # hide inherited Save and Cancel buttons
    @button.buttonAndHandler(_(u'Save'), condition=lambda form: False)
    def handleSave(self, action):
        pass

    @button.buttonAndHandler(_(u'Cancel'), condition=lambda form: False)
    def cancel(self, action):
        pass



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from Acquisition import aq_inner
from plone.app.users.browser.account import AccountPanelSchemaAdapter
from plone.app.users.browser.interfaces import ILoginNameGenerator
from plone.app.users.browser.interfaces import IUserIdGenerator
from plone.app.users.browser.schemaeditor import getFromBaseSchema
from plone.app.users.schema import IAddUserSchema
from plone.app.users.schema import ICombinedRegisterSchema
from plone.app.users.schema import IRegisterSchema
from plone.app.users.utils import notifyWidgetActionExecutionError
from plone.app.users.utils import uuid_userid_generator
from plone.autoform.form import AutoExtensibleForm
from plone.protect import CheckAuthenticator
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.interfaces import ISecuritySchema
from Products.CMFPlone.interfaces import IUserGroupsSettingsSchema
from Products.CMFPlone.utils import get_portal
from Products.CMFPlone.utils import normalizeString
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.interfaces import DISPLAY_MODE
from zExceptions import Forbidden
from ZODB.POSException import ConflictError
from zope.component import getAdapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import provideAdapter
from zope.component import queryUtility
from zope.schema import getFieldNames

import logging
import six


# Number of retries for creating a user id like bob-jones-42:
RENAME_AFTER_CREATION_ATTEMPTS = 100


def getRegisterSchema():
    portal = get_portal()
    schema = getattr(portal, '_v_register_schema', None)
    if schema is None:
        portal._v_register_schema = schema = getFromBaseSchema(
            ICombinedRegisterSchema,
            form_name=u'On Registration'
        )
        # as schema is a generated supermodel,
        # needed adapters can only be registered at run time
        provideAdapter(AccountPanelSchemaAdapter, (IPloneSiteRoot,), schema)
    return schema


class BaseRegistrationForm(AutoExtensibleForm, form.Form):
    """Form to be used as base for Register and Add User forms."""
    label = u""
    description = u""
    formErrorsMessage = _('There were errors.')
    ignoreContext = True
    enableCSRFProtection = True

    # this attribute indicates if user was successfully registered
    _finishedRegister = False

    @property
    def schema(self):
        return getRegisterSchema()

    def _get_security_settings(self):
        """Return security settings from the registry."""
        registry = getUtility(IRegistry)
        return registry.forInterface(ISecuritySchema, prefix="plone")

    def render(self):
        if self._finishedRegister:
            return self.context.unrestrictedTraverse('registered')()

        return super(BaseRegistrationForm, self).render()

    def updateFields(self):
        """Fields are dynamic in this form, to be able to handle
        different join styles.
        """
        settings = self._get_security_settings()
        use_email_as_login = settings.use_email_as_login

        # Finally, let autoform process the schema and any FormExtenders do
        # their thing
        super(BaseRegistrationForm, self).updateFields()

        if use_email_as_login:
            self.fields['email'].field.description = _(
                u'help_email_creation_for_login', default=u"Enter an email "
                "address. This will be your login name. We respect your "
                "privacy, and will not give the address away to any third "
                "parties or expose it anywhere.")
            del self.fields['username']
        else:
            self.fields['email'].field.description = _(
                u'help_email_creation',
                default=u"Enter an email address. This is necessary in case "
                        u"the password is lost. We respect your privacy, and "
                        u"will not give the address away to any third parties "
                        u"or expose it anywhere."
            )

        # Change the password description based on PAS Plugin The user needs a
        # list of instructions on what kind of password is required.  We'll
        # reuse password errors as instructions e.g. "Must contain a letter and
        # a number".  Assume PASPlugin errors are already translated
        if self.fields.get('password', None):
            registration = getToolByName(self.context, 'portal_registration')
            err_str = registration.testPasswordValidity('')
            if err_str:
                msg = _(
                    u'help_password_creation_with_errors',
                    default=u'Enter your new password. ${errors}',
                    mapping=dict(errors=err_str)
                )
                self.fields['password'].field.description = msg

    def generate_user_id(self, data):
        """Generate a user id from data.

        We try a few options for coming up with a good user id:

        1. We query a utility, so integrators can register a hook to
           generate a user id using their own logic.

        2. If use_uuid_as_userid is set in the registry, we
           generate a uuid.

        3. If a username is given and we do not use email as login,
           then we simply return that username as the user id.

        4. We create a user id based on the full name, if that is
           passed.  This may result in an id like bob-jones-2.

        When the email address is used as login name, we originally
        used the email address as user id as well.  This has a few
        possible downsides, which are the main reasons for the new,
        pluggable approach:

        - It does not work for some valid email addresses.

        - Exposing the email address in this way may not be wanted.

        - When the user later changes his email address, the user id
          will still be his old address.  It works, but may be
          confusing.

        Another possibility would be to simply generate a uuid, but that
        is ugly.  We could certainly try that though: the big plus here
        would be that you then cannot create a new user with the same user
        id as a previously existing user if this ever gets removed.  If
        you would get the same id, this new user would get the same global
        and local roles, if those have not been cleaned up.

        When a user id is chosen, the 'user_id' key of the data gets
        set and the user id is returned.
        """
        generator = queryUtility(IUserIdGenerator)
        if generator:
            userid = generator(data)
            if userid:
                data['user_id'] = userid
                return userid

        settings = self._get_security_settings()
        if settings.use_uuid_as_userid:
            userid = uuid_userid_generator()
            data['user_id'] = userid
            return userid

        # We may have a username already.
        userid = data.get('username')
        if userid:
            # If we are not using email as login, then this user name is fine.
            if not settings.use_email_as_login:
                data['user_id'] = userid
                return userid

        # First get a default value that we can return if we cannot
        # find anything better.
        pas = getToolByName(self.context, 'acl_users')
        email = pas.applyTransform(data.get('email'))
        default = data.get('username') or email or ''
        data['user_id'] = default
        fullname = data.get('fullname')
        if not fullname:
            return default
        userid = normalizeString(fullname)
        # First check that this is a valid member id, regardless of
        # whether a member with this id already exists or not.  We
        # access an underscore attribute of the registration tool, so
        # we take a precaution in case this is ever removed as an
        # implementation detail.
        registration = getToolByName(self.context, 'portal_registration')
        if hasattr(registration, '_ALLOWED_MEMBER_ID_PATTERN'):
            if not registration._ALLOWED_MEMBER_ID_PATTERN.match(userid):
                # If 'bob-jones' is not good then 'bob-jones-1' will not
                # be good either.
                return default
        if registration.isMemberIdAllowed(userid):
            data['user_id'] = userid
            return userid
        # Try bob-jones-1, bob-jones-2, etc.
        idx = 1
        while idx <= RENAME_AFTER_CREATION_ATTEMPTS:
            new_id = "%s-%d" % (userid, idx)
            if registration.isMemberIdAllowed(new_id):
                data['user_id'] = new_id
                return new_id
            idx += 1

        # We cannot come up with a nice id, so we simply return the default.
        return default

    def generate_login_name(self, data):
        """Generate a login name from data.

        Usually the login name and user id are the same, but this is
        not necessarily true.  When using the email address as login
        name, we may have a different user id, generated by calling
        the generate_user_id method.

        We try a few options for coming up with a good login name:

        1. We query a utility, so integrators can register a hook to
           generate a login name using their own logic.

        2. If a username is given and we do not use email as login,
           then we simply return that username as the login name.

        3. When using email as login, we use the email address.

        In all cases, we call PAS.applyTransform on the login name, if
        that is defined.  This is a recent addition to PAS, currently
        under development.

        When a login name is chosen, the 'login_name' key of the data gets
        set and the login name is returned.
        """
        pas = getToolByName(self.context, 'acl_users')
        generator = queryUtility(ILoginNameGenerator)
        if generator:
            login_name = generator(data)
            if login_name:
                login_name = pas.applyTransform(login_name)
                data['login_name'] = login_name
                return login_name

        # We may have a username already.
        login_name = data.get('username')
        login_name = pas.applyTransform(login_name)
        data['login_name'] = login_name
        settings = self._get_security_settings()
        # If we are not using email as login, then this user name is fine.
        if not settings.use_email_as_login:
            return login_name

        # We use email as login.
        login_name = data.get('email')
        login_name = pas.applyTransform(login_name)
        data['login_name'] = login_name
        return login_name

    # Actions validators
    def validate_registration(self, action, data):
        """Specific business logic for this join form.  Note: all this logic
        was taken directly from the old validate_registration.py script in
        Products/CMFPlone/skins/plone_login/join_form_validate.vpy
        """

        # CSRF protection
        CheckAuthenticator(self.request)

        registration = getToolByName(self.context, 'portal_registration')

        error_keys = [
            error.field.getName()
            for error
            in action.form.widgets.errors
        ]

        form_field_names = [f for f in self.fields]

        portal = getUtility(ISiteRoot)

        # passwords should match
        if 'password' in form_field_names:
            assert 'password_ctl' in form_field_names
            # Skip this check if password fields already have an error
            if not ('password' in error_keys or 'password_ctl' in error_keys):
                password = data.get('password')
                password_ctl = data.get('password_ctl')
                if password != password_ctl:
                    err_str = _(u'Passwords do not match.')
                    notifyWidgetActionExecutionError(action,
                                                     'password', err_str)
                    notifyWidgetActionExecutionError(action,
                                                     'password_ctl', err_str)

        # Password field checked against RegistrationTool
        if 'password' in form_field_names:
            # Skip this check if password fields already have an error
            if 'password' not in error_keys:
                password = data.get('password')
                if password:
                    # Use PAS to test validity
                    err_str = registration.testPasswordValidity(password)
                    if err_str:
                        notifyWidgetActionExecutionError(action,
                                                         'password', err_str)

        settings = self._get_security_settings()
        if settings.use_email_as_login:
            username_field = 'email'
        else:
            username_field = 'username'

        # The term 'username' is not clear.  It may be the user id or
        # the login name.  So here we try to be explicit.

        # Generate a nice user id and store that in the data.
        user_id = self.generate_user_id(data)
        # Generate a nice login name and store that in the data.
        login_name = self.generate_login_name(data)

        # Do several checks to see if the user id and the login name
        # are valid.
        #
        # Skip these checks if username was already in error list.
        #
        # Note that if we cannot generate a unique user id, it is not
        # necessarily the fault of the username field, but it
        # certainly is the most likely cause in a standard Plone
        # setup.

        # check if username is valid
        # Skip this check if username was already in error list
        if username_field not in error_keys:
            # user id may not be the same as the portal id.
            if user_id == portal.getId():
                err_str = _(u"This username is reserved. Please choose a "
                            "different name.")
                notifyWidgetActionExecutionError(action,
                                                 username_field, err_str)

        # Check if user id is allowed by the member id pattern.
        if username_field not in error_keys:
            if not registration.isMemberIdAllowed(user_id):
                err_str = _(u"The login name you selected is already in use "
                            "or is not valid. Please choose another.")
                notifyWidgetActionExecutionError(action,
                                                 username_field, err_str)

        if username_field not in error_keys:
            # Check the uniqueness of the login name, not only when
            # use_email_as_login is true, but always.
            pas = getToolByName(self, 'acl_users')
            results = pas.searchUsers(name=login_name, exact_match=True)
            if results:
                err_str = _(u"The login name you selected is already in use "
                            "or is not valid. Please choose another.")
                notifyWidgetActionExecutionError(action,
                                                 username_field, err_str)

        if 'password' in form_field_names and 'password' not in error_keys:
            # Admin can either set a password or mail the user (or both).
            if not (data['password'] or data['mail_me']):
                err_str = _('msg_no_password_no_mail_me',
                            default=u"You must set a password or choose to "
                            "send an email.")

                # set error on password field
                notifyWidgetActionExecutionError(action, 'password', err_str)
                notifyWidgetActionExecutionError(action, 'mail_me', err_str)

    @button.buttonAndHandler(
        _(u'label_register', default=u'Register'), name='register'
    )
    def action_join(self, action):
        data, errors = self.extractData()

        # extra password validation
        self.validate_registration(action, data)

        if action.form.widgets.errors:
            self.status = self.formErrorsMessage
            return

        self.handle_join_success(data)

        # XXX Return somewhere else, depending on what
        # handle_join_success returns?
        came_from = self.request.form.get('came_from')
        if came_from:
            utool = getToolByName(self.context, 'portal_url')
            if utool.isURLInPortal(came_from):
                self.request.response.redirect(came_from)
                return ''

    def handle_join_success(self, data):
        # portal should be acquisition wrapped, this is needed for the schema
        # adapter below
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        registration = getToolByName(self.context, 'portal_registration')

        # user_id and login_name should be in the data, but let's be safe.
        user_id = data.get('user_id', data.get('username'))
        login_name = data.get('login_name', data.get('username'))
        if six.PY2:
            # I have seen a unicode user id.  I cannot reproduce it, but
            # let's make them strings, otherwise you run into trouble with
            # plone.session when trying to login.
            if isinstance(user_id, six.text_type):
                user_id = user_id.encode('utf8')
            if isinstance(login_name, six.text_type):
                login_name = login_name.encode('utf8')

        # Set the username for good measure, as some code may expect
        # it to exist and contain the user id.
        data['username'] = user_id

        # The login name may already be in the form, but not
        # necessarily, for example when using email as login.  This is
        # at least needed for logging in immediately when password
        # reset is bypassed.  We need the login name here, not the
        # user id.
        self.request.form['form.username'] = login_name

        password = data.get('password') or registration.generatePassword()
        if six.PY2 and isinstance(password, six.text_type):
            password = password.encode('utf8')

        try:
            registration.addMember(user_id, password, REQUEST=self.request)
        except (AttributeError, ValueError) as err:
            logging.exception(err)
            IStatusMessage(self.request).addStatusMessage(err, type="error")
            self._finishedRegister = False
            return

        if user_id != login_name:
            # The user id differs from the login name.  Set the login
            # name correctly.
            pas = getToolByName(self.context, 'acl_users')
            pas.updateLoginName(user_id, login_name)

        # set member properties
        self.applyProperties(user_id, data)

        settings = self._get_security_settings()
        self._finishedRegister = True
        if data.get('mail_me') or (not settings.enable_user_pwd_choice and
                                   not data.get('password')):
            # We want to validate the email address (users cannot
            # select their own passwords on the register form) or the
            # admin has explicitly requested to send an email on the
            # 'add new user' form.
            try:
                # When all goes well, this call actually returns the
                # rendered mail_password_response template.  As a side
                # effect, this removes any status messages added to
                # the request so far, as they are already shown in
                # this template.
                response = registration.registeredNotify(user_id)
                return response
            except ConflictError:
                # Let Zope handle this exception.
                raise
            except Exception as err:
                logging.exception(err)
                ctrlOverview = getMultiAdapter((portal, self.request),
                                               name='overview-controlpanel')
                mail_settings_correct = not ctrlOverview.mailhost_warning()
                if mail_settings_correct:
                    # The email settings are correct, so the most
                    # likely cause of an error is a wrong email
                    # address.  We remove the account:
                    # Remove the account:
                    self.context.acl_users.userFolderDelUsers(
                        [user_id], REQUEST=self.request)
                    self._finishedRegister = False
                    IStatusMessage(self.request).addStatusMessage(
                        _(u'status_fatal_password_mail',
                          default=u"Failed to create your account: we were "
                          "unable to send instructions for setting a password "
                          "to your email address: ${address}",
                          mapping={u'address': data.get('email', '')}),
                        type='error')
                else:
                    # This should only happen when an admin registers
                    # a user.  The admin should have seen a warning
                    # already, but we warn again for clarity.
                    IStatusMessage(self.request).addStatusMessage(
                        _(u'status_nonfatal_password_mail',
                          default=u"This account has been created, but we "
                          "were unable to send instructions for setting a "
                          "password to this email address: ${address}",
                          mapping={u'address': data.get('email', '')}),
                        type='warning')

    def applyProperties(self, userid, data):
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        mt = getToolByName(self.context, 'portal_membership')
        member = mt.getMemberById(userid)

        # cache adapters
        adapters = {}

        # Set any fields that are simply properties for the new user, rather
        # than fields to help create the new user
        register_fields = getFieldNames(IRegisterSchema) + \
            getFieldNames(IAddUserSchema)
        for k, value in data.items():
            # skip fields not available in the schema
            if k in ['login_name', 'user_id']:
                continue

            # skip fields that are handled exclusively on user registration and
            # are not part of personal information form
            if k in register_fields:
                continue

            # get schema adapter
            schema = self.fields[k].field.interface
            if schema in adapters:
                adapter = adapters[schema]
            else:
                adapters[schema] = adapter = getAdapter(portal, schema)
                adapter.context = member
                adapter.schema = schema

            # finally set value
            setattr(adapter, k, value)


class RegistrationForm(BaseRegistrationForm):
    """Dynamically get fields from user data, through admin config settings.
    """
    label = _(u'heading_registration_form', default=u'Registration form')
    description = u""
    template = ViewPageTemplateFile('register_form.pt')

    @property
    def showForm(self):
        """The form should not be displayed to the user if the system is
           incapable of sending emails and email validation is switched on
           (users are not allowed to select their own passwords).
        """
        portal = getUtility(ISiteRoot)
        ctrlOverview = getMultiAdapter((portal, self.request),
                                       name='overview-controlpanel')

        settings = self._get_security_settings()
        # hide form if mailhost_warning == True and
        # enable_user_pwd_choice == False
        return not (ctrlOverview.mailhost_warning() and
                    not settings.enable_user_pwd_choice)

    def updateFields(self):
        if not self.showForm:
            # We do not want to spend time calculating fields that
            # will never get displayed.
            return

        super(RegistrationForm, self).updateFields()
        defaultFields = field.Fields(self.fields)

        # Can the user actually set his/her own password?
        settings = self._get_security_settings()
        if not settings.enable_user_pwd_choice:
            # No? Remove the password fields.
            defaultFields = defaultFields.omit('password', 'password_ctl')
        else:
            # The portal is not interested in validating emails, and
            # the user is not interested in getting an email with a
            # link to set his password if he can set this password in
            # the current form already.
            defaultFields = defaultFields.omit('mail_me')

        self.fields = defaultFields

    def updateWidgets(self):
        if not self.showForm:
            # We do not want to spend time calculating widgets that
            # will never get displayed.
            return

        super(RegistrationForm, self).updateWidgets()
        settings = self._get_security_settings()
        if not settings.enable_user_pwd_choice:
            # Show a message indicating that a password reset link
            # will be mailed to the user.
            widget = self.widgets['mail_me']
            widget.mode = DISPLAY_MODE
            widget.value = ['selected']
            widget.label = _(
                u'label_password_change_mail',
                default=u"A URL will be generated and e-mailed to you; "
                        u"follow the link to reach a page where you can "
                        u"change your password and complete the registration "
                        u"process.")
            widget.terms = None
            widget.updateTerms()


class AddUserForm(BaseRegistrationForm):
    label = _(u'heading_add_user_form', default=u'Add New User')
    description = u""
    template = ViewPageTemplateFile('newuser_form.pt')

    def updateFields(self):
        super(AddUserForm, self).updateFields()
        defaultFields = field.Fields(self.fields)

        # The mail_me field needs special handling depending on the
        # enable_user_pwd_choice setting and on the correctness of the mail
        # settings.
        portal = getUtility(ISiteRoot)
        ctrlOverview = getMultiAdapter((portal, self.request),
                                       name='overview-controlpanel')
        mail_settings_correct = not ctrlOverview.mailhost_warning()
        if mail_settings_correct:
            # Make the password fields optional: either specify a
            # password or mail the user (or both).  The validation
            # will check that at least one of the options is chosen.
            defaultFields['password'].field.required = False
            defaultFields['password_ctl'].field.required = False
            settings = self._get_security_settings()
            defaultFields['mail_me'].field.default =\
                not settings.enable_user_pwd_choice

        # Append the manager-focused fields
        user_group_settings = getAdapter(
            aq_inner(self.context),
            IUserGroupsSettingsSchema,
        )
        many_groups = user_group_settings.many_groups
        if not many_groups:
            allFields = defaultFields + field.Fields(IAddUserSchema)
            allFields['groups'].widgetFactory = CheckBoxFieldWidget
        else:
            allFields = defaultFields
        self.fields = allFields

    def updateWidgets(self):
        super(AddUserForm, self).updateWidgets()

        # set display mode for mail_me field if no mailhost is configured
        portal = getUtility(ISiteRoot)
        ctrlOverview = getMultiAdapter(
            (portal, self.request), name='overview-controlpanel'
        )
        mail_settings_correct = not ctrlOverview.mailhost_warning()
        if not mail_settings_correct:
            widget = self.widgets['mail_me']
            widget.mode = DISPLAY_MODE
            widget.value = ['selected']
            widget.label = _(
                u'label_cant_mail_password_reset',
                default=u"Normally we would offer to send the user an email "
                        u"with instructions to set a password on completion "
                        u"of this form. But this site does not have a valid "
                        u"email setup. You can fix this in the Mail settings."
            )
            widget.terms = None
            widget.updateTerms()

    @button.buttonAndHandler(
        _(u'label_register', default=u'Register'), name='register'
    )
    def action_join(self, action):
        data, errors = self.extractData()

        # extra password validation
        self.validate_registration(action, data)

        if action.form.widgets.errors:
            self.status = self.formErrorsMessage
            return

        self.handle_join_success(data)

        if not self._finishedRegister:
            return

        portal_groups = getToolByName(self.context, 'portal_groups')
        user_id = data['user_id']
        is_zope_manager = getSecurityManager().checkPermission(
            ManagePortal,
            self.context,
        )
        try:
            # Add user to the selected group(s)
            if data.get('groups', None) is not None:
                for groupname in data['groups']:
                    group = portal_groups.getGroupById(groupname)
                    if 'Manager' in group.getRoles() and not is_zope_manager:
                        raise Forbidden
                    portal_groups.addPrincipalToGroup(user_id, groupname,
                                                      self.request)
        except (AttributeError, ValueError) as err:
            IStatusMessage(self.request).addStatusMessage(err, type="error")
            return

        IStatusMessage(self.request).addStatusMessage(
            _(u"User added."), type='info')
        self.request.response.redirect(
            self.context.absolute_url() +
            '/@@usergroup-userprefs?searchstring=' + user_id)




# -*- coding: utf-8 -*-
from plone.app.users.browser.membersearch import extractCriteriaFromRequest

import unittest


class TestMemberSearch(unittest.TestCase):

    def test_extract_criteria_from_request(self):
        data = {
            '_authenticator': u'ab4731...',
            'form.buttons.search': u'Search',
            'form.widgets.something': u'any form value',
            'form.widgets.roles-empty-marker': True,
        }

        result = extractCriteriaFromRequest(data)

        self.assertEqual(result, {'something': u'any form value'})



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_encode

import logging
import plone.app.users.browser.schemaeditor as ttw
import six

logger = logging.getLogger('plone.app.users.setuphandlers')

FILE = 'userschema.xml'


def import_schema(context):
    """Import TTW Schema
    """
    data = context.readDataFile(FILE)
    if data is None:
        return
    if six.PY3 and isinstance(data, bytes):
        data = data.decode('utf-8')
    ttw.applySchema(data)
    logger.info('Imported schema')


def export_schema(context):
    """Export TTW schema
    """
    schema = ttw.serialize_ttw_schema()
    context.writeDataFile(FILE, safe_encode(schema), 'text/xml')
    logger.info('Exported schema')



# -*- coding: utf-8 -*-
from plone.app.users.browser.account import AccountPanelForm
from plone.app.users.browser.account import AccountPanelSchemaAdapter
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import getFSVersionTuple
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import Interface
from zope.schema import Choice


try:
    import plone.app.event  # noqa
    HAS_PAE = True
except ImportError:
    HAS_PAE = False

try:
    import plone.app.vocabularies.datetimerelated  # noqa
    HAS_DT_VOCAB = True
except ImportError:
    HAS_DT_VOCAB = False

PLONE5 = getFSVersionTuple()[0] >= 5


class IPersonalPreferences(Interface):
    """Provide schema for personalize form."""

    wysiwyg_editor = Choice(
        title=_(u'label_wysiwyg_editor', default=u'Wysiwyg editor'),
        description=_(
            u'help_wysiwyg_editor',
            default=u'Wysiwyg editor to use.'
        ),
        vocabulary="plone.app.vocabularies.AvailableEditors",
        required=False,
    )

    language = Choice(
        title=_(u'label_language', default=u'Language'),
        description=_(u'help_preferred_language', u'Your preferred language.'),
        vocabulary="plone.app.vocabularies.AvailableContentLanguages",
        required=False
    )

    if HAS_PAE and HAS_DT_VOCAB:
        timezone = Choice(
            title=_(u'label_timezone', default=u'Time zone'),
            description=_(u'help_timezone', default=u'Your time zone'),
            vocabulary='plone.app.vocabularies.AvailableTimezones',
            required=False,
        )
    elif HAS_PAE:
        timezone = Choice(
            title=_(u'label_timezone', default=u'Time zone'),
            description=_(u'help_timezone', default=u'Your time zone'),
            vocabulary='plone.app.vocabularies.Timezones',
            required=False,
        )


class PersonalPreferencesPanelAdapter(AccountPanelSchemaAdapter):
    schema = IPersonalPreferences


class PersonalPreferencesPanel(AccountPanelForm):
    """Implementation of personalize form that uses z3c.form."""

    form_name = _(u'legend_personal_details', u'Personal Details')
    schema = IPersonalPreferences

    @property
    def description(self):
        userid = self.request.form.get('userid')
        mt = getToolByName(self.context, 'portal_membership')
        if userid and (userid != mt.getAuthenticatedMember().getId()):
            # editing someone else's profile
            return _(
                u'description_preferences_form_otheruser',
                default='Personal settings for $name',
                mapping={'name': userid}
            )
        else:
            # editing my own profile
            return _(
                u'description_my_preferences',
                default='Your personal settings.'
            )

    def updateWidgets(self):
        super(PersonalPreferencesPanel, self).updateWidgets()

        self.widgets['language'].noValueMessage = _(
            u"vocabulary-missing-single-value-for-edit",
            u"Language neutral (site default)"
        )
        self.widgets['wysiwyg_editor'].noValueMessage = _(
            u"vocabulary-available-editor-novalue",
            u"Use site default"
        )

    def __call__(self):
        self.request.set('disable_border', 1)
        return super(PersonalPreferencesPanel, self).__call__()


class PersonalPreferencesConfiglet(PersonalPreferencesPanel):
    """Control panel version of the personal preferences panel"""
    template = ViewPageTemplateFile('account-configlet.pt')



# -*- coding: utf-8 -*-
# Note: test setup somehow fails when only tests from this file are run.
from plone.app.users.browser.interfaces import IUserIdGenerator
from plone.app.users.browser.register import BaseRegistrationForm
from plone.app.users.tests.base import BaseTestCase
from plone.app.users.utils import uuid_userid_generator
from zope.component import getSiteManager


class TestGenerateUserId(BaseTestCase):

    def test_custom_generator(self):
        """Test if a custom user id generator overrides the default
        behavior.
        """

        def one_generator(data):
            return 'one'

        sm = getSiteManager(context=self.portal)
        sm.registerUtility(one_generator, provided=IUserIdGenerator)
        form = BaseRegistrationForm(self.portal, {})

        data = {}
        self.assertEqual(form.generate_user_id(data), 'one')
        self.assertEqual(data.get('user_id'), 'one')

        data = {'username': 'joe',
                'fullname': 'Joe User',
                'email': 'joe@example.org'}
        self.assertEqual(form.generate_user_id(data), 'one')
        self.assertEqual(data.get('user_id'), 'one')

    def test_default(self):
        """Test generating user_id with no custom user id generator and
        the default security settings.
        """
        form = BaseRegistrationForm(self.portal, {})
        data = {}
        self.assertEqual(form.generate_user_id(data), '')
        self.assertEqual(data.get('user_id'), '')

        # An explicit username is taken by default.
        data = {'username': 'joe',
                'fullname': 'Joe User',
                'email': 'joe@example.org'}
        self.assertEqual(form.generate_user_id(data), 'joe')
        self.assertEqual(data.get('user_id'), 'joe')

        # When no username is there, we try a normalized fullname.
        data = {'fullname': 'Joe User',
                'email': 'joe@example.org'}
        self.assertEqual(form.generate_user_id(data), 'joe-user')
        self.assertEqual(data.get('user_id'), 'joe-user')

        # With no fullname, we take the email.
        data = {'email': 'Joe@example.org'}
        self.assertEqual(form.generate_user_id(data), 'Joe@example.org')
        self.assertEqual(data.get('user_id'), 'Joe@example.org')

    def test_use_email_as_login_has_fullname(self):
        """"Test generating a user id if the use_email_as_login setting is
        enabled and full name is provided.
        """
        self.security_settings.use_email_as_login = True
        form = BaseRegistrationForm(self.portal, {})

        data = {}
        self.assertEqual(form.generate_user_id(data), '')
        self.assertEqual(data.get('user_id'), '')

        data = {'fullname': 'Joe User',
                'email': 'joe@example.org'}
        self.assertEqual(form.generate_user_id(data), 'joe-user')
        self.assertEqual(data.get('user_id'), 'joe-user')

    def test_use_email_as_login_no_fullname(self):
        """"Test generating a user id if the use_email_as_login setting is
        enabled and full name is not provided.
        """
        self.security_settings.use_email_as_login = True
        form = BaseRegistrationForm(self.portal, {})

        data = {}
        self.assertEqual(form.generate_user_id(data), '')
        self.assertEqual(data.get('user_id'), '')

        data = {'email': 'joe@example.org'}
        self.assertEqual(form.generate_user_id(data), 'joe@example.org')
        self.assertEqual(data.get('user_id'), 'joe@example.org')

    def test_use_email_as_login_no_fullname_uppercase_email(self):
        """"Test generating a user id if the use_email_as_login setting is
        enabled and full name is not provided, with an uppercase e-mail.
        """
        self.security_settings.use_email_as_login = True
        form = BaseRegistrationForm(self.portal, {})

        data = {}
        self.assertEqual(form.generate_user_id(data), '')
        self.assertEqual(data.get('user_id'), '')

        data = {'email': 'Joe@Example.org'}
        self.assertEqual(form.generate_user_id(data), 'joe@example.org')
        self.assertEqual(data.get('user_id'), 'joe@example.org')

    def test_use_uuid_as_userid_enabled(self):
        """Test generating a user id if the use_uuid_as_userid setting is
        enabled.
        """
        self.security_settings.use_uuid_as_userid = True
        form = BaseRegistrationForm(self.portal, {})

        data = {}
        user_id = form.generate_user_id(data)
        self.assertEqual(data.get('user_id'), user_id)
        self.assertEqual(len(data.get('user_id')),
                         len(uuid_userid_generator()))

        data = {'username': 'joe',
                'fullname': 'Joe User',
                'email': 'joe@example.org'}
        user_id = form.generate_user_id(data)
        self.assertNotEqual(user_id, 'joe')
        self.assertEqual(data.get('user_id'), user_id)
        self.assertEqual(len(user_id), len(uuid_userid_generator()))

        # Calling it twice should give a different result, as every
        # call to the uuid generator should be unique.
        self.assertNotEqual(form.generate_user_id(data),
                            form.generate_user_id(data))



# -*- coding: utf-8 -*-
from plone.app.users.browser.userdatapanel import getUserDataSchema
from plone.app.users.setuphandlers import export_schema
from plone.app.users.setuphandlers import import_schema
from plone.app.users.testing import PLONE_APP_USERS_FUNCTIONAL_TESTING
from plone.app.users.tests.base import BaseTestCase
from plone.namedfile.field import NamedBlobImage
from Products.CMFCore.utils import getToolByName
from Products.GenericSetup.tests.common import DummyExportContext
from Products.GenericSetup.tests.common import DummyImportContext
from zope import schema


class TestImport(BaseTestCase):

    def setUp(self):
        super(TestImport, self).setUp()
        xml = """<model xmlns:lingua="http://namespaces.plone.org/supermodel/lingua" xmlns:users="http://namespaces.plone.org/supermodel/users" xmlns:form="http://namespaces.plone.org/supermodel/form" xmlns:i18n="http://xml.zope.org/namespaces/i18n" xmlns:security="http://namespaces.plone.org/supermodel/security" xmlns:marshal="http://namespaces.plone.org/supermodel/marshal" xmlns="http://namespaces.plone.org/supermodel/schema" i18n:domain="plone">
  <schema name="member-fields">
    <field name="home_page" type="zope.schema.URI" users:forms="In User Profile">
      <description i18n:translate="help_homepage">
          The URL for your external home page, if you have one.
      </description>
      <required>False</required>
      <title i18n:translate="label_homepage">Home Page</title>
    </field>
    <field name="description" type="zope.schema.Text" users:forms="In User Profile">
      <description i18n:translate="help_biography">
          A short overview of who you are and what you do. Will be displayed
          on your author page, linked from the items you create.
      </description>
      <required>False</required>
      <title i18n:translate="label_biography">Biography</title>
    </field>
    <field name="location" type="zope.schema.TextLine" users:forms="In User Profile">
      <description i18n:translate="help_location">
          Your location - either city and country - or in
          a company setting, where your office is located.
      </description>
      <required>False</required>
      <title i18n:translate="label_biography">Location</title>
    </field>
    <field name="portrait" type="plone.namedfile.field.NamedBlobImage" users:forms="In User Profile">
      <description i18n:translate="help_portrait">
          To add or change the portrait: click the "Browse" button; select a
          picture of yourself. Recommended image size is 75 pixels wide by
          100 pixels tall.
      </description>
      <required>False</required>
      <title i18n:translate="label_portrait">Portrait</title>
    </field>
    <field name="birthdate" type="zope.schema.Date" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Birthdate</title>
    </field>
    <field name="another_date" type="zope.schema.Datetime" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Another date</title>
    </field>
    <field name="age" type="zope.schema.Int" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Age</title>
    </field>
    <field name="department" type="zope.schema.Choice" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Department</title>
      <values>
        <element>Marketing</element>
        <element>Production</element>
        <element>HR</element>
      </values>
    </field>
    <field name="skills" type="zope.schema.Set" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Skills</title>
      <value_type type="zope.schema.Choice">
        <values>
          <element>Programming</element>
          <element>Management</element>
        </values>
      </value_type>
    </field>
    <field name="pi" type="zope.schema.Float" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Pi</title>
    </field>
    <field name="vegetarian" type="zope.schema.Bool" users:forms="In User Profile">
      <description/>
      <required>False</required>
      <title>Vegetarian</title>
    </field>
  </schema>
</model>
"""
        context = DummyImportContext(self.portal, purge=False)
        context._files = {'userschema.xml': xml}
        import_schema(context)

    def test_import(self):
        user_schema = getUserDataSchema()
        pm = getToolByName(self.portal, "portal_memberdata")
        member_properties = pm.propertyIds()

        self.assertIn("home_page", user_schema)
        self.assertTrue(isinstance(user_schema['home_page'], schema.URI))
        self.assertIn("home_page", member_properties)
        self.assertEqual(pm.getPropertyType('home_page'), 'text')

        self.assertIn("description", user_schema)
        self.assertTrue(isinstance(user_schema['description'], schema.Text))
        self.assertIn("description", member_properties)
        self.assertEqual(pm.getPropertyType('description'), 'text')

        self.assertIn("location", user_schema)
        self.assertTrue(isinstance(user_schema['location'], schema.TextLine))
        self.assertIn("location", member_properties)
        self.assertEqual(pm.getPropertyType('location'), 'string')

        self.assertIn("portrait", user_schema)
        self.assertTrue(isinstance(user_schema['portrait'], NamedBlobImage))
        # image fields are not handled as memberdata property,
        # it is handled directly in portal_membership
        self.assertNotIn("portrait", member_properties)

        self.assertIn("birthdate", user_schema)
        self.assertTrue(isinstance(user_schema['birthdate'], schema.Date))
        self.assertIn("birthdate", member_properties)
        self.assertEqual(pm.getPropertyType('birthdate'), 'date')

        self.assertIn("another_date", user_schema)
        self.assertTrue(isinstance(user_schema['another_date'], schema.Datetime))
        self.assertIn("another_date", member_properties)
        self.assertEqual(pm.getPropertyType('another_date'), 'date')

        self.assertIn("age", user_schema)
        self.assertTrue(isinstance(user_schema['age'], schema.Int))
        self.assertIn("age", member_properties)
        self.assertEqual(pm.getPropertyType('age'), 'int')

        self.assertIn("department", user_schema)
        self.assertTrue(isinstance(user_schema['department'], schema.Choice))
        self.assertIn("department", member_properties)
        self.assertEqual(pm.getPropertyType('department'), 'string')

        self.assertIn("skills", user_schema)
        self.assertTrue(isinstance(user_schema['skills'], schema.Set))
        self.assertIn("skills", member_properties)
        self.assertEqual(pm.getPropertyType('skills'), 'lines')

        self.assertIn("pi", user_schema)
        self.assertTrue(isinstance(user_schema['pi'], schema.Float))
        self.assertIn("pi", member_properties)
        self.assertEqual(pm.getPropertyType('pi'), 'float')

        self.assertIn("vegetarian", user_schema)
        self.assertTrue(isinstance(user_schema['vegetarian'], schema.Bool))
        self.assertIn("vegetarian", member_properties)
        self.assertEqual(pm.getPropertyType('vegetarian'), 'boolean')

    def test_export(self):
        context = DummyExportContext(self.portal)
        export_schema(context)
        self.assertEqual('userschema.xml', context._wrote[0][0])
        self.assertIn(b'field name="home_page"', context._wrote[0][1])
        self.assertIn(b'field name="description"', context._wrote[0][1])
        self.assertIn(b'field name="location"', context._wrote[0][1])
        self.assertIn(b'field name="portrait"', context._wrote[0][1])
        self.assertIn(b'field name="birthdate"', context._wrote[0][1])
        self.assertIn(b'field name="another_date"', context._wrote[0][1])
        self.assertIn(b'field name="age"', context._wrote[0][1])
        self.assertIn(b'field name="department"', context._wrote[0][1])
        self.assertIn(b'field name="skills"', context._wrote[0][1])
        self.assertIn(b'field name="pi"', context._wrote[0][1])
        self.assertIn(b'field name="vegetarian"', context._wrote[0][1])



# -*- coding: utf-8 -*-
from plone.app.users.schema import IRegisterSchema
from plone.app.users.schema import IUserDataSchema
from plone.app.users.schema import SCHEMA_ANNOTATION
from plone.app.users.schema import SCHEMATA_KEY
from plone.schemaeditor.browser.schema.listing import SchemaListing
from plone.schemaeditor.browser.schema.traversal import SchemaContext
from plone.supermodel import loadString
from plone.supermodel.model import finalizeSchemas
from plone.supermodel.model import Model
from plone.supermodel.model import SchemaClass
from plone.supermodel.parser import IFieldMetadataHandler
from plone.supermodel.serializer import serialize
from plone.supermodel.utils import ns
from plone.z3cform.layout import FormWrapper
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.utils import get_portal
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.annotation.interfaces import IAnnotations
from zope.component import getGlobalSiteManager
from zope.interface import implementer
from zope.interface import Interface

import copy
import logging
import re


USERS_NAMESPACE = 'http://namespaces.plone.org/supermodel/users'
USERS_PREFIX = 'users'
SPLITTER = '_//_'

ALLOWED_FIELDS = [
    u'zope.schema._bootstrapfields.TextLine',
    u'zope.schema._bootstrapfields.Text',
    u'zope.schema._bootstrapfields.Bool',
    u'zope.schema._bootstrapfields.Int',
    u'zope.schema._field.Float',
    u'zope.schema._field.Set',
    u'zope.schema._field.Choice',
    u'zope.schema._field.Date',
    u'zope.schema._field.Datetime',
    u'plone.namedfile.field.NamedBlobImage',
    u'zope.schema._field.URI',
]
field_type_mapping = {
    "ProtectedEmail": 'string',
    "ProtectedTextLine": 'string',
    "TextLine": 'string',
    "Text": 'text',
    "Bool": 'boolean',
    "Int": 'int',
    "Float": 'float',
    "Set": 'lines',
    "Choice": 'string',
    "Date": 'date',
    "Datetime": 'date',
    "NamedBlobImage": '__portrait__',
    "URI": 'text',
}

DEFAULT_VALUES = {
    'text': '',
    'int': 0,
    'float': 0.0,
    'boolean': False,
}

re_flags = re.S | re.U | re.X


def log(message,
        level='info',
        id='plone.app.users.browser.schemaeditor'):
    logger = logging.getLogger(id)
    getattr(logger, level)(message)


class IMemberFieldValidator(Interface):
    """Base marker for field validators"""


class IMemberSchemaContext(Interface):
    """ """


class SchemaListingPage(FormWrapper):

    form = SchemaListing
    index = ViewPageTemplateFile('schema_layout.pt')


@implementer(IMemberSchemaContext)
class MemberSchemaContext(SchemaContext):

    label = _(u"Edit Member Form Fields")

    def __init__(self, context, request):
        self.fieldsWhichCannotBeDeleted = ['fullname', 'email']
        self.showSaveDefaults = False
        self.enableFieldsets = False
        self.allowedFields = ALLOWED_FIELDS

        schema = getFromBaseSchema(IUserDataSchema)
        super(MemberSchemaContext, self).__init__(
            schema,
            request,
            name=SCHEMATA_KEY,
            title=_(u"Member Fields"),
        )


def updateSchema(object, event):
    snew_schema = serialize_ttw_schema(object.schema)
    applySchema(snew_schema)


def applySchema(snew_schema):
    site = get_portal()

    # get the old schema (currently stored in the annotation)
    old_schema = get_ttw_edited_schema()

    # check if more than 2 image fields:
    if snew_schema.count('NamedBlobImage') > 1:
        site.plone_utils.addPortalMessage(
            _(u'One image field maximum.'), 'error')
        return

    # store the current schema in the annotation
    set_schema(snew_schema)

    # load the new schema
    new_schema = get_ttw_edited_schema()

    # update portal_memberdata properties
    pm = getToolByName(site, "portal_memberdata")
    existing = pm.propertyIds()
    for field_id in [a for a in new_schema]:
        field_type = field_type_mapping.get(
            new_schema[field_id].__class__.__name__,
            None)
        if not field_type:
            log('Unsupported field: %s (%s)' % (
                field_id,
                new_schema[field_id].__class__.__name__))
            continue
        if field_type == '__portrait__':
            continue
        if field_id in existing:
            pm._delProperty(field_id)
        pm._setProperty(
            field_id,
            DEFAULT_VALUES.get(field_type, ''),
            field_type)

    if old_schema:
        to_remove = [field_id
                     for field_id in [a for a in old_schema]
                     if field_id not in [a for a in new_schema]]
        for field_id in to_remove:
            field_type = field_type_mapping.get(
                old_schema[field_id].__class__.__name__,
                None)
            if field_type == '__portrait__':
                continue
            pm._delProperty(field_id)

    invalidateSchemasInCache(site)


def get_ttw_edited_schema():
    data = get_schema()
    if data:
        ttwschema = load_ttw_schema(data)
        if ttwschema is None:
            return ''
        return ttwschema
    return ''


@implementer(IFieldMetadataHandler)
class UsersMetadataSchemaExporter(object):
    """Support the security: namespace in model definitions.
    """
    namespace = ns = USERS_NAMESPACE
    prefix = USERS_PREFIX
    if_attrs = (
        'min', 'max', 'order',
        'min_length', 'max_length',
        'required',
    )

    def read(self, fieldNode, schema, field):
        for attr in self.if_attrs:
            value = self.load(
                fieldNode.get(ns(attr, self.ns), None))
            if value is not None:
                setattr(field, attr, value)

    def write(self, fieldNode, schema, field):
        for attr in self.if_attrs:
            value = getattr(field, attr, None)
            if value is not None:
                v = self.serialize(value)
                fieldNode.set(ns(attr, self.ns), v)

    def load(self, value):
        listre = re.compile('(?P<type>list|set|tuple)'
                            ':(?P<list>.*)', re_flags)
        ltypes = {
            'list': list,
            'set': set,
            'tuple': tuple,
        }
        if isinstance(value, basestring):
            listm = listre.search(value)
            if value.startswith("int:"):
                value = int(value.split('int:')[1])
            elif listm:
                i = listm.groupdict()
                try:
                    tp = i["type"]
                    value = i["list"].split(SPLITTER)
                    if tp not in ['list']:
                        value = ltypes[tp](value)
                except:
                    value = []
            else:
                value = {"bool:true": True,
                         "bool:false": False}.get(value.lower(), value)
        return value

    def serialize(self, value):
        if isinstance(value, bool):
            value = value and "bool:true" or "bool:false"
        elif isinstance(value, (list, set, tuple)):
            value = u"%s:%s" % (type(value).__name__, SPLITTER.join(value))
        elif value is not None:
            value = u"int:%s" % unicode(value)
        return value


def is_serialisable_field(field):
    ret = False
    if field.__class__.__name__ in field_type_mapping:
        ret = True
    else:
        raise TypeError("type not serializable %s" % field.__class__.__name__)
    return ret


def serialize_ttw_schema(schema=None):
    if not schema:
        schema = get_ttw_edited_schema()
    bfields = [a for a in IUserDataSchema]
    attrs = {}
    for name in schema:
        f = schema[name]
        if is_serialisable_field(f) and name not in bfields:
            attrs[name] = f
    smember = SchemaClass(SCHEMATA_KEY, attrs=attrs)
    finalizeSchemas(smember)
    model = Model({SCHEMATA_KEY: smember})
    sschema = serialize(model)
    return sschema


def load_ttw_schema(string=None):
    if not string:
        string = get_schema()
    schema = loadString(string).schemata.get(SCHEMATA_KEY, None)
    return schema


def get_schema(site=None):
    if site is None:
        site = get_portal()
    annotations = IAnnotations(site)
    return annotations.get(SCHEMA_ANNOTATION, '')


def set_schema(string, site=None):
    if site is None:
        site = get_portal()
    annotations = IAnnotations(site)
    annotations[SCHEMA_ANNOTATION] = string


def invalidateSchemasInCache(portal):

    gsm = getGlobalSiteManager()

    schema = getattr(portal, '_v_register_schema', None)
    if schema is not None:
        from .account import AccountPanelSchemaAdapter
        gsm.unregisterAdapter(
            AccountPanelSchemaAdapter,
            (IPloneSiteRoot,),
            schema
        )
    portal._v_register_schema = None

    schema = getattr(portal, '_v_userdata_schema', None)
    if schema is not None:
        from .userdatapanel import UserDataPanelAdapter
        gsm.unregisterAdapter(
            UserDataPanelAdapter,
            (IPloneSiteRoot,),
            schema
        )
    portal._v_userdata_schema = None

    # kill volatile attributes in all threads
    portal._p_changed = 1


def getFromBaseSchema(baseSchema, form_name=None):
    attrs = copySchemaAttrs(baseSchema, form_name)
    ttwschema = get_ttw_edited_schema()
    if ttwschema:
        attrs.update(copySchemaAttrs(ttwschema, form_name))
    schema = SchemaClass(SCHEMATA_KEY,
                         bases=(baseSchema,),
                         attrs=attrs)
    finalizeSchemas(schema)
    return schema


def copySchemaAttrs(schema, form_name):
    return dict([
        (a, copy.deepcopy(schema[a]))
        for a in schema
        if field_in_form(schema[a], form_name)
    ])


default_fields = list(IUserDataSchema.names()) + list(IRegisterSchema.names())


def field_in_form(field, form_name=None):
    if form_name is None:
        return True
    if field.__name__ in default_fields:
        return True
    forms_selection = getattr(field, 'forms_selection', [])
    return form_name in forms_selection



# -*- coding: utf-8 -*-
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.users.browser.account import AccountPanelForm
from plone.app.users.browser.account import AccountPanelSchemaAdapter
from plone.app.users.browser.schemaeditor import getFromBaseSchema
from plone.app.users.schema import IUserDataSchema
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.interfaces import ISecuritySchema
from Products.CMFPlone.utils import get_portal
from Products.CMFPlone.utils import set_own_login_name
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zExceptions import NotFound
from zope.component import getUtility
from zope.component import provideAdapter

try:
    from html import escape
except ImportError:
    from cgi import escape


class UserDataPanelAdapter(AccountPanelSchemaAdapter):
    """One does not simply set portrait, email might be used to login with.
    """

    @property
    def schema(self):
        # prevent infinite recursion when accessing the schema via bypassing
        # __getattr__ of self
        try:
            return object.__getattribute__(self, '_schema')
        except AttributeError:
            object.__setattr__(self, '_schema', getUserDataSchema())
        return object.__getattribute__(self, '_schema')

    @schema.setter
    def schema(self, value):
        self._schema = value

    def get_email(self):
        return self._getProperty('email')

    def set_email(self, value):
        registry = getUtility(IRegistry)
        security_settings = registry.forInterface(
            ISecuritySchema, prefix="plone")
        if security_settings.use_email_as_login:
            mt = getToolByName(self.context, 'portal_membership')
            if self.context.getId() == mt.getAuthenticatedMember().getId():
                set_own_login_name(self.context, value)
            else:
                pas = getToolByName(self.context, 'acl_users')
                pas.updateLoginName(self.context.getId(), value)
        return self._setProperty('email', value)

    email = property(get_email, set_email)


class UserDataPanel(AccountPanelForm):

    form_name = _(u'User Data Form')
    enableCSRFProtection = True

    @property
    def schema(self):
        schema = getUserDataSchema()
        return schema

    @property
    def description(self):
        userid = self.request.form.get('userid')
        mt = getToolByName(self.context, 'portal_membership')
        if userid and (userid != mt.getAuthenticatedMember().getId()):
            # editing someone else's profile
            return _(
                u'description_personal_information_form_otheruser',
                default='Change personal information for $name',
                mapping={'name': escape(userid)}
            )
        else:
            # editing my own profile
            return _(
                u'description_personal_information_form',
                default='Change your personal information'
            )

    def __call__(self):
        userid = self.request.form.get('userid')
        if userid:
            mt = getToolByName(self.context, 'portal_membership')
            if mt.getMemberById(userid) is None:
                raise NotFound('User does not exist.')
        self.request.set('disable_border', 1)
        return super(UserDataPanel, self).__call__()


def getUserDataSchema():
    portal = get_portal()
    schema = getattr(portal, '_v_userdata_schema', None)
    if schema is None:
        portal._v_userdata_schema = schema = getFromBaseSchema(
            IUserDataSchema,
            form_name=u'In User Profile'
        )
        # as schema is a generated supermodel,
        # needed adapters can only be registered at run time
        provideAdapter(UserDataPanelAdapter, (IPloneSiteRoot,), schema)
        provideAdapter(UserDataPanelAdapter, (INavigationRoot,), schema)
    return schema


class UserDataConfiglet(UserDataPanel):
    """Control panel version of the userdata panel"""
    template = ViewPageTemplateFile('account-configlet.pt')



# -*- coding: utf-8 -*-
from plone.app.users.testing import PLONE_APP_USERS_ACCEPTANCE_TESTING
from plone.testing import layered

import os
import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    for testfile in os.listdir(
            os.path.join(os.path.dirname(__file__), "acceptance")):
        testfilepath = os.path.join("acceptance", testfile)
#        if not os.path.isdir(testfilepath) and testfile.endswith('.robot'):
#            suite.addTests([
#                layered(
#                    robotsuite.RobotTestSuite(
#                        testfilepath,
#                        noncritical=['fixme']),
#                    layer=PLONE_APP_USERS_ACCEPTANCE_TESTING),
#            ])
    return suite



# -*- coding: utf-8 -*-
from plone.app.users.tests.base import BaseTestCase
from plone.app.users.utils import uuid_userid_generator
from Products.CMFCore.utils import getToolByName

import transaction


class TestNewUser(BaseTestCase):

    def test_new_user_as_site_administrator(self):
        self.portal.acl_users._doAddUser(
            'siteadmin', 'secret', ['Site Administrator'], []
        )
        # make the user available
        transaction.commit()

        self.browser.addHeader('Authorization', 'Basic siteadmin:secret')
        self.browser.open('http://nohost/plone/new-user')
        self.browser.getControl('User Name').value = 'newuser'
        self.browser.getControl('Email').value = 'newuser@example.com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Site Administrators').selected = True
        self.browser.getControl('Register').click()

        # make sure the new user is in the Site Administrators group
        self.assertTrue(
            'Site Administrator' in
            self.portal.acl_users.getUserById('newuser').getRoles()
        )


class TestGenerateUserIdLoginName(BaseTestCase):
    """Test if the user id and user name are properly generated based on the
    security settings.
    """

    def setUp(self):
        super(TestGenerateUserIdLoginName, self).setUp()
        self.portal_url = self.portal.absolute_url()
        self.portal.acl_users._doAddUser(
            'siteadmin', 'secret', ['Site Administrator'], []
        )
        transaction.commit()
        self.browser.addHeader('Authorization', 'Basic siteadmin:secret')

    def test_uuid_disabled_email_as_login_disabled(self):
        self.security_settings.use_uuid_as_userid = False
        self.security_settings.use_email_as_login = False
        transaction.commit()

        # create a user
        self.browser.open('http://nohost/plone/@@new-user')
        self.browser.getControl('Full Name').value = 'New User'
        self.browser.getControl('User Name').value = 'newie'
        self.browser.getControl('Email').value = 'NewUser@Example.Com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Register').click()

        # user id should be set the same as user name
        pas = getToolByName(self.portal, 'acl_users')
        self.assertEquals(len(pas.searchUsers(name='newie')), 1)
        user = pas.getUser('newie')
        self.assertEquals(user.getId(), 'newie')
        self.assertEquals(user.getUserName(), 'newie')

    def test_uuid_disabled_email_as_login_enabled_no_full_name(self):
        self.security_settings.use_uuid_as_userid = False
        self.security_settings.use_email_as_login = True
        transaction.commit()

        # create a user
        self.browser.open('http://nohost/plone/@@new-user')
        self.browser.getControl('Email').value = 'newuser@example.com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Register').click()

        # Since full name is not provided, the user id is set based on the
        # e-mail, the same as the user name.
        pas = getToolByName(self.portal, 'acl_users')
        self.assertEquals(len(pas.searchUsers(name='newuser@example.com')), 1)
        self.assertEquals(len(pas.searchUsers(name='newuser@example.com')), 1)
        user = pas.getUser('newuser@example.com')
        self.assertEquals(user.getId(), 'newuser@example.com')
        self.assertEquals(user.getUserName(), 'newuser@example.com')

    def test_uuid_disabled_email_as_login_enabled_no_full_name_uppercase(self):
        self.security_settings.use_uuid_as_userid = False
        self.security_settings.use_email_as_login = True
        transaction.commit()

        # create a user
        self.browser.open('http://nohost/plone/@@new-user')
        self.browser.getControl('Email').value = 'NewUser@Example.Com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Register').click()

        # the user id is set based on the e-mail, which should be lowercased
        pas = getToolByName(self.portal, 'acl_users')
        self.assertEquals(len(pas.searchUsers(name='newuser@example.com')), 1)
        self.assertEquals(len(pas.searchUsers(name='NewUser@Example.Com')), 1)
        user = pas.getUser('newuser@Example.Com')
        self.assertEquals(user.getId(), 'newuser@example.com')
        self.assertEquals(user.getUserName(), 'newuser@example.com')

    def test_uuid_disabled_email_as_login_enabled_has_full_name(self):
        self.security_settings.use_uuid_as_userid = False
        self.security_settings.use_email_as_login = True
        transaction.commit()

        # create a user
        self.browser.open('http://nohost/plone/@@new-user')
        self.browser.getControl('Full Name').value = 'New User'
        self.browser.getControl('Email').value = 'NewUser@Example.Com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Register').click()

        # User id should be set based on the full name, user name should be
        # set based on the e-mail.
        pas = getToolByName(self.portal, 'acl_users')
        self.assertEquals(len(pas.searchUsers(name='newuser@example.com')), 1)
        self.assertEquals(len(pas.searchUsers(name='NewUser@Example.Com')), 1)
        user = pas.getUser('newuser@Example.Com')
        self.assertEquals(user.getId(), 'new-user')
        self.assertEquals(user.getUserName(), 'newuser@example.com')

    def test_uuid_enabled_email_as_login_disabled(self):
        self.security_settings.use_uuid_as_userid = True
        self.security_settings.use_email_as_login = False
        transaction.commit()

        # create a user
        self.browser.open('http://nohost/plone/@@new-user')
        self.browser.getControl('Full Name').value = 'New User'
        self.browser.getControl('User Name').value = 'newie'
        self.browser.getControl('Email').value = 'NewUser@Example.Com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Register').click()

        # uuid should be used for the user id
        pas = getToolByName(self.portal, 'acl_users')
        self.assertEquals(len(pas.searchUsers(name='newie')), 1)
        user = pas.getUser('newie')
        self.assertEquals(len(user.getId()), len(uuid_userid_generator()))
        self.assertNotEquals(user.getId(), 'newuser@example.com')
        self.assertNotEquals(user.getId(), 'newie')
        self.assertNotEquals(user.getId(), 'new-user')
        self.assertEquals(user.getUserName(), 'newie')

    def test_uuid_enabled_email_as_login_enabled(self):
        self.security_settings.use_uuid_as_userid = True
        self.security_settings.use_email_as_login = True
        transaction.commit()

        # create a user
        self.browser.open('http://nohost/plone/@@new-user')
        self.browser.getControl('Full Name').value = 'New User'
        self.browser.getControl('Email').value = 'NewUser@Example.Com'
        self.browser.getControl('Password').value = 'foobar'
        self.browser.getControl('Confirm password').value = 'foobar'
        self.browser.getControl('Register').click()

        # uuid should be used for the user id, user name should be based on
        # the e-mail
        pas = getToolByName(self.portal, 'acl_users')
        self.assertEquals(len(pas.searchUsers(name='newuser@example.com')), 1)
        self.assertEquals(len(pas.searchUsers(name='NewUser@Example.Com')), 1)
        user = pas.getUser('newuser@example.com')
        self.assertEquals(len(user.getId()), len(uuid_userid_generator()))
        self.assertNotEquals(user.getId(), 'newuser@example.com')
        self.assertNotEquals(user.getId(), 'newie')
        self.assertNotEquals(user.getId(), 'new-user')
        self.assertEquals(user.getUserName(), 'newuser@example.com')



# -*- coding: utf-8 -*-
from Products.CMFCore.interfaces import IMembershipTool
from plone.app.users.browser.account import AccountPanelSchemaAdapter
from plone.app.users.tests.base import BaseTestCase
from plone.testing import z2
from zope.interface import implementer
from zope.component import provideUtility
import unittest


@implementer(IMembershipTool)
class DummyPortalMembership(object):

    def __init__(self, allowed):
        self.allowed = allowed

    def getMemberById(self, id):
        return id

    def getAuthenticatedMember(self):
        return '(authenticated)'

    def checkPermission(self, permission, context):
        return self.allowed


class TestAccountPanelSchemaAdapter(unittest.TestCase):
    layer = z2.INTEGRATION_TESTING

    def test__init__no_userid(self):
        """Should edit current user."""
        provideUtility(DummyPortalMembership(False))
        adapter = AccountPanelSchemaAdapter(self.layer['request'])
        self.assertEqual('(authenticated)', adapter.context)

    def test__init__userid_in_request_form_for_non_manager(self):
        """Disallow for non-privileged users."""
        provideUtility(DummyPortalMembership(False))
        self.layer['request'].form['userid'] = 'bob'
        adapter = AccountPanelSchemaAdapter(self.layer['request'])
        self.assertEqual('(authenticated)', adapter.context)

    def test__init__userid_in_request_form_for_manager(self):
        """Should allow for privileged users."""
        provideUtility(DummyPortalMembership(True))
        self.layer['request'].form['userid'] = 'bob'
        adapter = AccountPanelSchemaAdapter(self.layer['request'])
        self.assertEqual('bob', adapter.context)



# -*- coding: utf-8 -*-
from plone.autoform.form import AutoExtensibleForm
from plone.supermodel import model
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import form
from zope import schema


class IMemberSearchSchema(model.Schema):
    """Provide schema for member search."""

    model.fieldset(
        'extra',
        label=_(u'legend_member_search_criteria',
                default=u'User Search Criteria'),
        fields=['login', 'email', 'fullname']
    )

    login = schema.TextLine(
        title=_(u'label_name', default=u'Name'),
        description=_(
            u'help_search_name',
            default=u'Find users whose login name contain'),
        required=False,
    )
    email = schema.TextLine(
        title=_(u'label_email', default=u'Email'),
        description=_(
            u'help_search_email',
            default=u'Find users whose email address contain'),
        required=False,
    )
    fullname = schema.TextLine(
        title=_(u'label_fullname', default=u'Full Name'),
        description=_(
            u'help_search_fullname',
            default=u'Find users whose full names contain'),
        required=False,
    )
    # disabled: https://dev.plone.org/ticket/13862
    """
    directives.read_permission(roles='cmf.ManagePortal')
    directives.write_permission(roles='cmf.ManagePortal')
    directives.widget('roles', CheckBoxWidget)
    roles = schema.List(
        title=_(u'label_roles', default=u'Role(s)'),
        description=_(
            u'help_search_roles',
            default=u'Find users with all of the selected roles.'),
        required=False,
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Roles',
        ),
    )
    """


def extractCriteriaFromRequest(criteria):
    """Takes a dictionary of z3c.form data and sanitizes it to fit
    for a pas member search.
    """
    for key in ['_authenticator',
                'form.buttons.search',
                'form.widgets.roles-empty-marker', ]:
        if key in criteria:
            del criteria[key]
    for (key, value) in list(criteria.items()):
        if not value:
            del criteria[key]
        else:
            new_key = key.replace('form.widgets.', '')
            criteria[new_key] = value
            del criteria[key]

    return criteria


class MemberSearchForm(AutoExtensibleForm, form.Form):
    """This search form enables you to find users by specifying one or more
    search criteria.
    """

    schema = IMemberSearchSchema
    ignoreContext = True

    label = _(u'heading_member_search', default=u'Search for users')
    description = _(u'description_member_search',
                    default=u'This search form enables you to find users by '
                            u'specifying one or more search criteria.')
    template = ViewPageTemplateFile('membersearch_form.pt')
    enableCSRFProtection = True
    formErrorsMessage = _('There were errors.')

    submitted = False

    @button.buttonAndHandler(_(u'label_search', default=u'Search'),
                             name='search')
    def handleApply(self, action):
        request = self.request
        data, errors = self.extractData()

        if errors:
            self.status = self.formErrorsMessage
            return

        if request.get('form.buttons.search', None):
            self.submitted = True

            view = self.context.restrictedTraverse('@@pas_search')
            criteria = extractCriteriaFromRequest(self.request.form.copy())
            self.results = view.searchUsers(sort_by=u'fullname', **criteria)



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from .browser.schemaeditor import getFromBaseSchema
from plone.app.users.schema import ICombinedRegisterSchema
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import get_portal
from Products.CMFPlone.utils import normalizeString
from Products.CMFPlone.utils import safe_unicode
from zope.interface import implementer
from zope.schema import getFieldNames
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


# Define constants from the Join schema that should be added to the
# vocab of the join fields setting in usergroupssettings controlpanel.
JOIN_CONST = ['username', 'password', 'email', 'mail_me']


@implementer(IVocabularyFactory)
class UserRegistrationFieldsVocabulary(object):
    """Returns list of fields available for registration form.

    It gets fields from z3c.form adopted Registration form schema.
    FormExtender fields are not included.

      >>> from zope.component import queryUtility
      >>> from zope.schema.interfaces import IVocabularyFactory

      >>> name = 'plone.app.users.user_registration_fields'
      >>> util = queryUtility(IVocabularyFactory, name)

      >>> fields = util(None)
      >>> fields
      <zope.schema.vocabulary.SimpleVocabulary object at ...>

      >>> len(fields.by_token)
      10
      >>> values = [k.value for k in fields]
      >>> sorted(values)
      ['description', 'email', 'fullname', 'home_page', 'location', 'mail_me', 'password', 'password_ctl', 'portrait', 'username']

      >>> email = fields.by_token['email']
      >>> email.title, email.token, email.value
      ('email', 'email', 'email')

    """

    def __call__(self, context):
        # default list of Registration Form fields
        schema = getFromBaseSchema(ICombinedRegisterSchema)
        values = getFieldNames(schema)

        # make sure required minimum number of fields is present
        for val in JOIN_CONST:
            if val not in values:
                values.append(val)

        return SimpleVocabulary([SimpleTerm(v, v, v) for v in values])


UserRegistrationFieldsVocabularyFactory = UserRegistrationFieldsVocabulary()


@implementer(IVocabularyFactory)
class GroupIdVocabulary(object):
    """
    Return vocab of groups to add new user to.

      >>> from zope.component import queryUtility
      >>> from zope.schema.interfaces import IVocabularyFactory
      >>> from Products.CMFPlone.utils import get_portal
      >>> from Products.CMFCore.utils import getToolByName

      >>> groups_tool = getToolByName(get_portal(), 'portal_groups')
      >>> groups_tool.addGroup(
      ...     'fancygroup', [], [],
      ...     title='Group Title',
      ...     description="Group Description",
      ... )
      True

      >>> name = 'plone.app.users.group_ids'
      >>> util = queryUtility(IVocabularyFactory, name)

      >>> fields = util(get_portal())
      >>> fields
      <zope.schema.vocabulary.SimpleVocabulary object at ...>

      >>> [k.value for k in fields] # doctest: +NORMALIZE_WHITESPACE
      ['fancygroup', 'Reviewers', 'Site Administrators']
      >>> [k.title for k in fields] # doctest: +NORMALIZE_WHITESPACE
      ['Group Title (fancygroup)', 'Reviewers', 'Site Administrators']

    """

    def __call__(self, context):
        site = get_portal()
        groups_tool = getToolByName(site, 'portal_groups')
        is_zope_manager = getSecurityManager().checkPermission(
            ManagePortal, context)
        groups = groups_tool.listGroups()

        # Get group id, title tuples for each, omitting virtual group
        # 'AuthenticatedUsers'
        terms = []
        for g in groups:
            if g.id == 'AuthenticatedUsers':
                continue
            if 'Manager' in g.getRoles() and not is_zope_manager:
                continue

            group_title = safe_unicode(g.getGroupTitleOrName())
            if group_title != g.id:
                title = u'%s (%s)' % (group_title, g.id)
            else:
                title = group_title
            terms.append(SimpleTerm(g.id, g.id, title))

        # Sort by title
        terms.sort(key=lambda x: normalizeString(x.title))
        return SimpleVocabulary(terms)


GroupIdVocabularyFactory = GroupIdVocabulary()



# -*- coding: utf-8 -*-
"""Base class for flexible user registration test cases.

This is in a separate module because it's potentially useful to other
packages which register accountpanels. They should be able to import it
without the PloneTestCase.setupPloneSite() side effects.
"""
from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import aq_base
from OFS.Cache import Cacheable
from plone.app.testing import setRoles
from plone.app.testing import login
from plone.app.users.testing import PLONE_APP_USERS_FUNCTIONAL_TESTING
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from Products.CMFPlone.interfaces import ISecuritySchema
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.tests.utils import MockMailHost
from Products.MailHost.interfaces import IMailHost
from Products.PlonePAS.setuphandlers import activatePluginInterfaces
from Products.PluggableAuthService.interfaces.plugins import IValidationPlugin
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import classImplements
from transaction import commit
from zope.component import getSiteManager
from zope.component import getUtility

import unittest


class BaseTestCase(unittest.TestCase):
    """ base test case which adds amin user """

    layer = PLONE_APP_USERS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal.acl_users._doAddUser('admin', 'secret', ['Manager'], [])
        set_mock_mailhost(self.portal)
        self.membership = self.portal.portal_membership
        self.security_settings = get_security_settings()

        self.browser = Browser(self.layer['app'])
        self.request = self.layer['request']

    def tearDown(self):
        login(self.portal, 'admin')
        unset_mock_mailhost(self.portal)
        pas_instance = self.portal.acl_users
        plugin = getattr(pas_instance, 'test', None)
        if plugin is not None:
            plugins = pas_instance._getOb('plugins')
            plugins.deactivatePlugin(IValidationPlugin, 'test')
            # plugins.deactivatePlugin(IPropertiesPlugin, 'test')
            pas_instance.manage_delObjects('test')

    def test_nothing(self):
        """ Add a dummy test here, so the base class 'passes'. """

# Dummy password validation PAS plugin


class DeadParrotPassword(BasePlugin, Cacheable):
    meta_type = 'Test Password Strength Plugin'
    security = ClassSecurityInfo()

    def __init__(self, id, title=None):
        self._id = self.id = id
        self.title = title

    security.declarePrivate('validateUserInfo')

    def validateUserInfo(self, user, set_id, set_info):
        errors = []
        if set_info and set_info.get('password', None) is not None:
            password = set_info['password']
            if password.count('dead') or password == '':
                errors = [{'id': 'password', 'error': u'Must not be dead'}]
            else:
                errors = []
        return errors


# Helper methods used in doctests

def setMailHost():
    registry = getUtility(IRegistry)
    mail_settings = registry.forInterface(IMailSchema, prefix='plone')
    mail_settings.smtp_host = u'localhost'
    mail_settings.email_from_address = 'admin@foo.com'
    commit()


def unsetMailHost():
    registry = getUtility(IRegistry)
    mail_settings = registry.forInterface(IMailSchema, prefix='plone')
    mail_settings.smtp_host = u''
    mail_settings.email_from_address = ''
    commit()


def activateDefaultPasswordPolicy(portal):
    uf = portal.acl_users
    for policy in uf.objectIds(['Default Plone Password Policy']):
        activatePluginInterfaces(portal, policy)


def addParrotPasswordPolicy(portal):
    # remove default policy
    uf = portal.acl_users
    for policy in uf.objectIds(['Default Plone Password Policy']):
        uf.plugins.deactivatePlugin(IValidationPlugin, policy)

    obj = DeadParrotPassword('test')
    uf._setObject(obj.getId(), obj)
    obj = uf[obj.getId()]
    activatePluginInterfaces(portal, obj.getId())

    # portal = getUtility(ISiteRoot)
    plugins = uf._getOb('plugins')
    validators = plugins.listPlugins(IValidationPlugin)
    assert validators
    commit()


classImplements(DeadParrotPassword, IValidationPlugin)


def get_security_settings():
    registry = getUtility(IRegistry)
    return registry.forInterface(ISecuritySchema, prefix="plone")


def set_mock_mailhost(portal):
    portal._original_MailHost = portal.MailHost
    portal.MailHost = mailhost = MockMailHost('MailHost')
    sm = getSiteManager(context=portal)
    sm.unregisterUtility(provided=IMailHost)
    sm.registerUtility(mailhost, provided=IMailHost)


def unset_mock_mailhost(portal):
    portal.MailHost = portal._original_MailHost
    sm = getSiteManager(context=portal)
    sm.unregisterUtility(provided=IMailHost)
    sm.registerUtility(aq_base(portal._original_MailHost), provided=IMailHost)



# -*- coding: utf-8 -*-
from plone.formwidget.namedfile.widget import NamedImageWidget
from plone.namedfile.interfaces import INamedImageField
from plone.schema.email import Email
from plone.schemaeditor.fields import FieldFactory
from plone.schemaeditor.interfaces import IFieldFactory
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.RegistrationTool import EmailAddressInvalid
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope import schema
from zope.component import adapter
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Interface
from ZTUtils import make_query


SCHEMA_ANNOTATION = "plone.app.users.schema"
# must match the browser view name !
SCHEMATA_KEY = "member-fields"


def checkEmailAddress(value):
    portal = getUtility(ISiteRoot)

    reg_tool = getToolByName(portal, 'portal_registration')
    if value and reg_tool.isValidEmail(value):
        pass
    else:
        # TODO: check if there is a z3c validator
        raise EmailAddressInvalid
    return True


class ProtectedTextLine(schema.TextLine):
    """ TextLine field which cannot be edited via shema editor
    """
    pass


class ProtectedEmail(Email):
    """ Email field which cannot be edited via shema editor
    """
    pass


@implementer(IFieldFactory)
class NotEditableFieldFactory(FieldFactory):

    title = _(u'(protected)')

    def protected(self, field):
        return True


FullnameFieldFactory = NotEditableFieldFactory(
    ProtectedTextLine,
    _(u'label_full_name', default=u'Full Name'),
)

EmailFieldFactory = NotEditableFieldFactory(
    ProtectedEmail,
    _(u'label_email', default=u'Email'),
)


class IUserDataSchema(Interface):
    """
    """

    fullname = ProtectedTextLine(
        title=_(u'label_full_name', default=u'Full Name'),
        description=_(u'help_full_name_creation',
                      default=u"Enter full name, e.g. John Smith."),
        required=False)

    email = ProtectedEmail(
        title=_(u'label_email', default=u'Email'),
        description=u'We will use this address if you need to recover your '
                    u'password',
        required=True,
        constraint=checkEmailAddress,
    )


class IRegisterSchema(Interface):

    username = schema.ASCIILine(
        title=_(u'label_user_name', default=u'User Name'),
        description=_(
            u'help_user_name_creation_casesensitive',
            default=u"Enter a user name, usually something like 'jsmith'. "
                    u"No spaces or special characters. Usernames and "
                    u"passwords are case sensitive, make sure the caps lock "
                    u"key is not enabled. This is the name used to log in."
        )
    )

    password = schema.Password(
        title=_(u'label_password', default=u'Password'),
        description=_(u'help_password_creation',
                      default=u'Enter your new password.'))

    password_ctl = schema.Password(
        title=_(u'label_confirm_password',
                default=u'Confirm password'),
        description=_(u'help_confirm_password',
                      default=u"Re-enter the password. "
                      "Make sure the passwords are identical."))

    mail_me = schema.Bool(
        title=_(u'label_mail_password',
                default=u"Send a confirmation mail with a link to set the "
                u"password"),
        required=False,
        default=False)


class ICombinedRegisterSchema(IRegisterSchema, IUserDataSchema):
    """Collect all register fields under the same interface"""


class IAddUserSchema(Interface):

    groups = schema.List(
        title=_(u'label_add_to_groups',
                default=u'Add to the following groups:'),
        description=u'',
        required=False,
        value_type=schema.Choice(vocabulary='plone.app.users.group_ids'))


class PortraitWidget(NamedImageWidget):
    # Cheat around 2 bugs:
    # * You are not authenticated during traversal, so fetching
    #   the current user does not work.
    # * download_url won't append our querystring, so fetching
    #   another user's image does not work.
    @property
    def download_url(self):
        userid = self.request.form.get('userid')
        if not userid:
            mt = getToolByName(self.form.context, 'portal_membership')
            userid = mt.getAuthenticatedMember().getId()

        # anonymous
        if not userid:
            return None

        url = super(PortraitWidget, self).download_url
        if not url:
            return None

        return '%s?%s' % (url, make_query({'userid': userid}))


@implementer(IFieldWidget)
@adapter(INamedImageField, IFormLayer)
def PortraitFieldWidget(field, request):
    return FieldWidget(field, PortraitWidget(request))


class IRegistrationSettingsSchema(Interface):

    user_registration_fields = schema.Tuple(
        title=_(
            u'title_user_registration_fields',
            default=u'User registration fields'
        ),
        description=_(
            u"description_user_registration_fields",
            default=(u"Select the fields for the join form. Fields in the "
                     u"right box will be shown on the form, fields on the "
                     u"left are disabled. Use the left/right buttons to move "
                     u"a field from right to left (to disable it) and vice "
                     u"versa. Use the up/down buttons to change the order in "
                     u"which the fields appear on the form."),
        ),
        value_type=schema.Choice(
            vocabulary='plone.app.users.user_registration_fields'),
    )


class IUserSchemaProvider(Interface):

    pass



# -*- coding: utf-8 -*-
from plone.app.users.browser import schemaeditor
from plone.namedfile.field import NamedBlobImage
from plone.schemaeditor.interfaces import IEditableSchema
from plone.supermodel.model import finalizeSchemas
from plone.supermodel.model import Schema
from plone.supermodel.model import SchemaClass
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from zope import schema
from zope.interface import Interface

import copy
import logging


log = logging.getLogger("plone.app.users:upgrade")


def copySchemaAttrs(sch):
    fields = {}
    for id in sch:
        field = copy.deepcopy(sch[id])
        field.forms_selection = [u'In User Profile']
        fields[id] = field
    return fields


class IEmpty(Schema):
    pass


class IHomePageSchema(Interface):
    """
    """

    home_page = schema.TextLine(
        title=_(u'label_homepage', default=u'Home page'),
        description=_(u'help_homepage',
                      default=u"The URL for your external home page, "
                      "if you have one."),
        required=False)


class IDescriptionSchema(Interface):
    """
    """

    description = schema.Text(
        title=_(u'label_biography', default=u'Biography'),
        description=_(u'help_biography',
                      default=u"A short overview of who you are and what you "
                      "do. Will be displayed on your author page, linked "
                      "from the items you create."),
        required=False)


class ILocationSchema(Interface):
    """
    """

    location = schema.TextLine(
        title=_(u'label_location', default=u'Location'),
        description=_(u'help_location',
                      default=u"Your location - either city and "
                      "country - or in a company setting, where "
                      "your office is located."),
        required=False)


class IPortraitSchema(Interface):
    """
    """

    portrait = NamedBlobImage(
        title=_(u'label_portrait', default=u'Portrait'),
        description=_(
            u'help_portrait',
            default=u'To add or change the portrait: click the "Browse" '
                    u'button; select a picture of yourself. Recommended '
                    u'image size is 75 pixels wide by 100 pixels tall.'
        ),
        required=False)


def upgrade_to_ttw(context):
    # the new default schema only contains fullname and email fields
    # so we put the missing ones (home_page, description, location, portrait)
    # into the ttw schema
    if schemaeditor.get_schema() == '':
        finalizeSchemas(IEmpty)
        current_ttw = IEditableSchema(IEmpty)
    else:
        current_ttw = IEditableSchema(schemaeditor.load_ttw_schema())

    attrs = copySchemaAttrs(current_ttw.schema)
    current_fields = current_ttw.schema.names()
    pm = getToolByName(context, "portal_memberdata")
    existing = pm.propertyIds()

    if 'home_page' in existing and 'home_page' not in current_fields:
        attrs.update(copySchemaAttrs(IHomePageSchema))

    if 'description' in existing and 'description' not in current_fields:
        attrs.update(copySchemaAttrs(IDescriptionSchema))

    if 'location' in existing and 'location' not in current_fields:
        attrs.update(copySchemaAttrs(ILocationSchema))

    if 'portrait' in existing and 'portrait' not in current_fields:
        attrs.update(copySchemaAttrs(IPortraitSchema))

    sch = SchemaClass(schemaeditor.SCHEMATA_KEY,
        bases=(current_ttw.schema,),
        attrs=attrs
    )
    finalizeSchemas(sch)

    xml_model = schemaeditor.serialize_ttw_schema(sch)
    schemaeditor.set_schema(xml_model)
    log.info('Old member fields migrated into TTW schema')



# -*- coding: utf-8 -*-
# Note: test setup somehow fails when only tests from this file are run.
from plone.app.users.browser.interfaces import ILoginNameGenerator
from plone.app.users.browser.register import BaseRegistrationForm
from plone.app.users.tests.base import BaseTestCase
from zope.component import getSiteManager


class TestGenerateLoginName(BaseTestCase):

    def test_custom_generator(self):
        """Test if a custom login name generator overrides the default
        behavior.
        """
        sm = getSiteManager(context=self.portal)
        form = BaseRegistrationForm(self.portal, {})
        data = {'useme': 'me me me', 'username': 'frank'}

        sm.registerUtility(
            lambda data: data['useme'], provided=ILoginNameGenerator)

        self.assertEqual(form.generate_login_name(data), 'me me me')
        self.assertEqual(data.get('login_name'), 'me me me')

    def test_custom_generator_empty(self):
        """Test that the username is used if a custom login name generator
        returns an empty value.
        """
        sm = getSiteManager(context=self.portal)
        form = BaseRegistrationForm(self.portal, {})
        data = {'useme': '', 'username': 'Frank'}

        sm.registerUtility(
            lambda data: data['useme'], provided=ILoginNameGenerator)

        self.assertEqual(form.generate_login_name(data), 'Frank')
        self.assertEqual(data.get('login_name'), 'Frank')

    def test_use_email_as_login_disabled(self):
        """Test generating user_id with no custom login name generator and
        with the use_email_as_login security setting disabled.
        """
        form = BaseRegistrationForm(self.portal, {})
        data = {'username': 'Frank'}
        self.security_settings.use_email_as_login = False

        self.assertEqual(form.generate_login_name(data), 'Frank')
        self.assertEqual(data.get('login_name'), 'Frank')

    def test_use_email_as_login_enabled(self):
        """Test generating user_id with no custom login name generator and
        with the use_email_as_login security setting enabled.
        """
        form = BaseRegistrationForm(self.portal, {})
        data = {'username': 'Frank', 'email': 'Frank@Test.com'}
        self.security_settings.use_email_as_login = True

        self.assertEqual(form.generate_login_name(data), 'frank@test.com')
        self.assertEqual(data.get('login_name'), 'frank@test.com')



# -*- coding: utf-8 -*-
from plone.app.users.schema import IRegistrationSettingsSchema
from plone.protect import CheckAuthenticator
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.orderedselect import OrderedSelectFieldWidget


class RegistrationControlPanel(form.Form):
    label = _("Users and Groups")
    description = _(u"Registration settings for this site.")
    form_name = _(u"Registration settings")
    enableCSRFProtection = True

    formErrorsMessage = _('There were errors.')
    template = ViewPageTemplateFile('memberregistration.pt')

    fields = field.Fields(IRegistrationSettingsSchema)
    fields['user_registration_fields'].widgetFactory = OrderedSelectFieldWidget

    def getContent(self):
        props = self.props()
        return {'user_registration_fields': props.getProperty(
            'user_registration_fields', [])}

    @button.buttonAndHandler(
        _(u'label_apply_changes', default=u'Apply Changes'),
        name='save'
    )
    def action_save(self, action):
        # CSRF protection
        CheckAuthenticator(self.request)

        data, errors = self.extractData()
        if errors:
            IStatusMessage(self.request).addStatusMessage(
                self.formErrorsMessage, type='error')
            return

        # save property
        if data['user_registration_fields'] != \
                self.getContent()['user_registration_fields']:
            props = self.props()
            props._updateProperty(
                'user_registration_fields',
                data['user_registration_fields']
            )
            msg = _("Changes saved.")
        else:
            msg = _("No changes made.")
        IStatusMessage(self.request).addStatusMessage(msg, type="info")

    # @button.buttonAndHandler(
    #     _(u'label_cancel', default=u'Cancel'), name='cancel'
    # )
    # def action_cancel(self, action):
    #     IStatusMessage(self.request).addStatusMessage(
    #         _("Changes canceled."), type="info"
    #     )
    #     url = getMultiAdapter(
    #         (self.context, self.request),
    #         name='absolute_url'
    #     )()
    #     self.request.response.redirect(url + '/@@overview-controlpanel')

    def updateActions(self):
        super(RegistrationControlPanel, self).updateActions()
        if self.actions and 'save' in self.actions:
            self.actions['save'].addClass('context')

    def props(self):
        pprop = getToolByName(self.context, 'portal_properties')
        return pprop.site_properties



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.users.browser.userdatapanel import UserDataPanel
from plone.app.users.testing import PLONE_APP_USERS_FUNCTIONAL_TESTING
from plone.app.users.tests.base import BaseTestCase
from zExceptions import NotFound
from zope.i18n import translate


class TestUserDataPanel(BaseTestCase):

    def test_regression(self):
        portal = self.layer['portal']
        request = self.layer['request']
        request.form.update({
            'userid': 'admin'
        })
        form = UserDataPanel(portal, request)
        description = translate(form.description, context=request)
        self.assertTrue('admin' in description)
        # form can be called without raising exception.
        self.assertTrue(form())

    def test_escape_html(self):
        portal = self.layer['portal']
        request = self.layer['request']
        request.form.update({
            'userid': 'admin<script>alert("userid")</script>'
        })
        form = UserDataPanel(portal, request)
        description = translate(form.description, context=request)
        self.assertTrue('<script>' not in description)
        self.assertRaises(NotFound, form)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '2.6.7'

long_description = u'{0}\n\n{1}'.format(
    open('README.rst').read(),
    open('CHANGES.rst').read()
)

extras_require = {
    'test': [
        'Products.MailHost',
        'plone.app.testing',
        'plone.keyring',
    ]
}

setup(
    name='plone.app.users',
    version=version,
    description='A package for all things users and groups related (specific '
                'to plone)',
    long_description=long_description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Plone :: 5.2',
        'Framework :: Plone :: Core',
        'Framework :: Zope2',
        'Framework :: Zope :: 4',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='Zope CMF Plone Users Groups',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.org/project/plone.app.users',
    license='GPL version 2',
    packages=find_packages(),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    extras_require=extras_require,
    install_requires=[
        'AccessControl',
        'Acquisition',
        'Products.CMFCore',
        'Products.CMFPlone >= 5.1a2',
        'Products.PlonePAS >= 5.0.1',
        'Products.statusmessages',
        'Zope2 >= 2.12.3',
        'plone.app.layout',
        'plone.autoform >= 1.2',
        'plone.formwidget.namedfile >= 1.0.3',
        'plone.namedfile',
        'plone.protect',
        'plone.schema',
        'plone.uuid',
        'setuptools',
        'six',
        'z3c.form',
        'zope.component',
        'zope.event',
        'zope.interface',
        'zope.schema',
    ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from plone.app.users.testing import PLONE_APP_USERS_FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import re
import six
import unittest


doc_tests = [
    'duplicate_email.rst',
    'email_login.rst',
    'flexible_user_registration.rst',
    'forms_navigationroot.rst',  # need to ask about content types layer
    'member_search.rst',
    'registration_forms.rst',  # working on it
    'password.rst',  # for later
    'personal_preferences.rst',
    'personal_preferences_prefs_user_details.rst',
    'userdata.rst',
    'userdata_prefs_user_details.rst',
    '../vocabularies.py',
]

optionflags = (
    doctest.ELLIPSIS |
    doctest.NORMALIZE_WHITESPACE |
    doctest.REPORT_ONLY_FIRST_FAILURE
)


class Py23DocChecker(doctest.OutputChecker):
    def check_output(self, want, got, optionflags):
        if six.PY2:
            got = got.replace('Unauthorized', 'zExceptions.unauthorized.Unauthorized')
            got = got.replace(':utf8:ustring', '')
            got = re.sub("u'(.*?)'", "'\\1'", got)
            want = re.sub("b'(.*?)'", "'\\1'", want)
        return doctest.OutputChecker.check_output(self, want, got, optionflags)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(
            doctest.DocFileSuite(
                'tests/{0}'.format(test_file),
                package='plone.app.users',
                optionflags=optionflags,
                checker=Py23DocChecker(),
            ),
            layer=PLONE_APP_USERS_FUNCTIONAL_TESTING)
        for test_file in doc_tests
    ])

    return suite



# -*- coding: utf-8 -*-
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.testing import applyProfile
from plone.app.testing import login
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing.layers import FunctionalTesting
from plone.app.users.browser.userdatapanel import UserDataPanel
from plone.app.users.testing import PLONE_APP_USERS_FUNCTIONAL_TESTING
from plone.testing import z2
from Products.CMFCore.utils import getToolByName
from transaction import commit
from zExceptions import NotFound
from zope.i18n import translate

import unittest


class WITHPAMLayer(PloneSandboxLayer):

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.multilingual:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        language_tool = getToolByName(portal, 'portal_languages')
        language_tool.addSupportedLanguage('fr')
        language_tool.addSupportedLanguage('it')
        setup_tool = SetupMultilingualSite()
        setup_tool.setupSite(portal)


WITHPAM_FIXTURE = WITHPAMLayer()

WITHPAM_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(WITHPAM_FIXTURE,),
    name="PloneAppUsersWithPAMLayer:Functional")


class TestUserDataPanelWithPAM(unittest.TestCase):

    layer = WITHPAM_FUNCTIONAL_TESTING

    def test_pam(self):
        browser = z2.Browser(self.layer['app'])
        browser.addHeader('Authorization', 'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD))
        browser.open('http://nohost/plone/fr/@@personal-information')
        self.assertIn(
            'Saisissez votre nom complet, par exemple Jean Dupont.',
            browser.contents,
        )
